# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

import numpy as np

from ...testing import utils

from .. import nilearn as iface
from ...pipeline import engine as pe

import pytest
import numpy.testing as npt

no_nilearn = True
try:
    __import__('nilearn')
    no_nilearn = False
except ImportError:
    pass


@pytest.mark.skipif(no_nilearn, reason="the nilearn library is not available")
class TestSignalExtraction():

    filenames = {
        'in_file': 'fmri.nii',
        'label_files': 'labels.nii',
        '4d_label_file': '4dlabels.nii',
        'out_file': 'signals.tsv'
    }
    labels = ['CSF', 'GrayMatter', 'WhiteMatter']
    global_labels = ['GlobalSignal'] + labels

    @pytest.fixture(autouse=True, scope='class')
    def setup_class(self, tmpdir_factory):
        tempdir = tmpdir_factory.mktemp("test")
        self.orig_dir = tempdir.chdir()
        utils.save_toy_nii(self.fake_fmri_data, self.filenames['in_file'])
        utils.save_toy_nii(self.fake_label_data, self.filenames['label_files'])

    def test_signal_extract_no_shared(self):
        # run
        iface.SignalExtraction(
            in_file=self.filenames['in_file'],
            label_files=self.filenames['label_files'],
            class_labels=self.labels,
            incl_shared_variance=False).run()
        # assert
        self.assert_expected_output(self.labels, self.base_wanted)

    def test_signal_extr_bad_label_list(self):
        # run
        with pytest.raises(ValueError):
            iface.SignalExtraction(
                in_file=self.filenames['in_file'],
                label_files=self.filenames['label_files'],
                class_labels=['bad'],
                incl_shared_variance=False).run()

    def test_signal_extr_equiv_4d_no_shared(self):
        self._test_4d_label(
            self.base_wanted,
            self.fake_equiv_4d_label_data,
            incl_shared_variance=False)

    def test_signal_extr_4d_no_shared(self):
        # set up & run & assert
        self._test_4d_label(
            self.fourd_wanted,
            self.fake_4d_label_data,
            incl_shared_variance=False)

    def test_signal_extr_global_no_shared(self):
        # set up
        wanted_global = [[-4. / 6], [-1. / 6], [3. / 6], [-1. / 6], [-7. / 6]]
        for i, vals in enumerate(self.base_wanted):
            wanted_global[i].extend(vals)

        # run
        iface.SignalExtraction(
            in_file=self.filenames['in_file'],
            label_files=self.filenames['label_files'],
            class_labels=self.labels,
            include_global=True,
            incl_shared_variance=False).run()

        # assert
        self.assert_expected_output(self.global_labels, wanted_global)

    def test_signal_extr_4d_global_no_shared(self):
        # set up
        wanted_global = [[3. / 8], [-3. / 8], [1. / 8], [-7. / 8], [-9. / 8]]
        for i, vals in enumerate(self.fourd_wanted):
            wanted_global[i].extend(vals)

        # run & assert
        self._test_4d_label(
            wanted_global,
            self.fake_4d_label_data,
            include_global=True,
            incl_shared_variance=False)

    def test_signal_extr_shared(self):
        # set up
        wanted = []
        for vol in range(self.fake_fmri_data.shape[3]):
            volume = self.fake_fmri_data[:, :, :, vol].flatten()
            wanted_row = []
            for reg in range(self.fake_4d_label_data.shape[3]):
                region = self.fake_4d_label_data[:, :, :, reg].flatten()
                wanted_row.append(
                    (volume * region).sum() / (region * region).sum())

            wanted.append(wanted_row)
        # run & assert
        self._test_4d_label(wanted, self.fake_4d_label_data)

    def test_signal_extr_traits_valid(self):
        ''' Test a node using the SignalExtraction interface.
        Unlike interface.run(), node.run() checks the traits
        '''
        # run
        node = pe.Node(
            iface.SignalExtraction(
                in_file=os.path.abspath(self.filenames['in_file']),
                label_files=os.path.abspath(self.filenames['label_files']),
                class_labels=self.labels,
                incl_shared_variance=False),
            name='SignalExtraction')
        node.run()

        # assert
        # just checking that it passes trait validations

    def _test_4d_label(self,
                       wanted,
                       fake_labels,
                       include_global=False,
                       incl_shared_variance=True):
        # set up
        utils.save_toy_nii(fake_labels, self.filenames['4d_label_file'])

        # run
        iface.SignalExtraction(
            in_file=self.filenames['in_file'],
            label_files=self.filenames['4d_label_file'],
            class_labels=self.labels,
            incl_shared_variance=incl_shared_variance,
            include_global=include_global).run()

        wanted_labels = self.global_labels if include_global else self.labels

        # assert
        self.assert_expected_output(wanted_labels, wanted)

    def assert_expected_output(self, labels, wanted):
        with open(self.filenames['out_file'], 'r') as output:
            got = [line.split() for line in output]
            labels_got = got.pop(0)  # remove header
            assert labels_got == labels
            assert len(got) == self.fake_fmri_data.shape[
                3], 'num rows and num volumes'
            # convert from string to float
            got = [[float(num) for num in row] for row in got]
            for i, time in enumerate(got):
                assert len(labels) == len(time)
                for j, segment in enumerate(time):
                    npt.assert_almost_equal(segment, wanted[i][j], decimal=1)


# dj: self doesnt have orig_dir at this point, not sure how to change it.
# should work without it
#    def teardown_class(self):
#        self.orig_dir.chdir()

    fake_fmri_data = np.array([[[[2, -1, 4, -2, 3], [4, -2, -5, -1, 0]],
                                [[-2, 0, 1, 4, 4], [-5, 3, -3, 1, -5]]],
                               [[[2, -2, -1, -2, -5], [3, 0, 3, -5, -2]],
                                [[-4, -2, -2, 1, -2], [3, 1, 4, -3, -2]]]])

    fake_label_data = np.array([[[1, 0], [3, 1]], [[2, 0], [1, 3]]])

    fake_equiv_4d_label_data = np.array(
        [[[[1., 0., 0.], [0., 0., 0.]], [[0., 0., 1.], [1., 0., 0.]]],
         [[[0., 1., 0.], [0., 0., 0.]], [[1., 0., 0.], [0., 0., 1.]]]])

    base_wanted = [[-2.33333, 2, .5], [0, -2, .5], [-.3333333, -1, 2.5],
                   [0, -2, .5], [-1.3333333, -5, 1]]

    fake_4d_label_data = np.array([[[[0.2, 0.3, 0.5], [0.1, 0.1, 0.8]],
                                    [[0.1, 0.3, 0.6], [0.3, 0.4, 0.3]]],
                                   [[[0.2, 0.2, 0.6], [0., 0.3, 0.7]],
                                    [[0.3, 0.3, 0.4], [0.3, 0.4, 0.3]]]])

    fourd_wanted = [[-5.0652173913, -5.44565217391, 5.50543478261], [
        -7.02173913043, 11.1847826087, -4.33152173913
    ], [-19.0869565217, 21.2391304348,
        -4.57608695652], [5.19565217391, -3.66304347826, -1.51630434783],
                    [-12.0, 3., 0.5]]



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import Label2Label


def test_Label2Label_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        copy_inputs=dict(),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hemisphere=dict(
            argstr='--hemi %s',
            mandatory=True,
        ),
        out_file=dict(
            argstr='--trglabel %s',
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source=['source_label'],
            name_template='%s_converted',
        ),
        registration_method=dict(
            argstr='--regmethod %s',
            usedefault=True,
        ),
        source_label=dict(
            argstr='--srclabel %s',
            extensions=None,
            mandatory=True,
        ),
        source_sphere_reg=dict(
            extensions=None,
            mandatory=True,
        ),
        source_subject=dict(
            argstr='--srcsubject %s',
            mandatory=True,
        ),
        source_white=dict(
            extensions=None,
            mandatory=True,
        ),
        sphere_reg=dict(
            extensions=None,
            mandatory=True,
        ),
        subject_id=dict(
            argstr='--trgsubject %s',
            mandatory=True,
            usedefault=True,
        ),
        subjects_dir=dict(),
        white=dict(
            extensions=None,
            mandatory=True,
        ),
    )
    inputs = Label2Label.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Label2Label_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = Label2Label.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import LocalBistat


def test_LocalBistat_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        automask=dict(
            argstr='-automask',
            xor=['weight_file'],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file1=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        in_file2=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        mask_file=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        neighborhood=dict(
            argstr="-nbhd '%s(%s)'",
            mandatory=True,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            keep_extension=True,
            name_source='in_file1',
            name_template='%s_bistat',
            position=0,
        ),
        outputtype=dict(),
        stat=dict(
            argstr='-stat %s...',
            mandatory=True,
        ),
        weight_file=dict(
            argstr='-weight %s',
            extensions=None,
            xor=['automask'],
        ),
    )
    inputs = LocalBistat.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_LocalBistat_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = LocalBistat.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..confounds import NonSteadyStateDetector


def test_NonSteadyStateDetector_inputs():
    input_map = dict(in_file=dict(
        extensions=None,
        mandatory=True,
    ), )
    inputs = NonSteadyStateDetector.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_NonSteadyStateDetector_outputs():
    output_map = dict(n_volumes_to_discard=dict(), )
    outputs = NonSteadyStateDetector.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..imagelabelcombine import ImageLabelCombine


def test_ImageLabelCombine_inputs():
    input_map = dict(
        InputLabelMap_A=dict(
            argstr='%s',
            extensions=None,
            position=-3,
        ),
        InputLabelMap_B=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        OutputLabelMap=dict(
            argstr='%s',
            hash_files=False,
            position=-1,
        ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        first_overwrites=dict(argstr='--first_overwrites ', ),
    )
    inputs = ImageLabelCombine.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ImageLabelCombine_outputs():
    output_map = dict(OutputLabelMap=dict(
        extensions=None,
        position=-1,
    ), )
    outputs = ImageLabelCombine.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The dtitk module provides classes for interfacing with the `DTITK
<http://dti-tk.sourceforge.net/pmwiki/pmwiki.php>`_ command line tools.

These are the base tools for working with DTITK.
Preprocessing tools are found in dtitk/preprocess.py
Registration tools are found in dtitk/registration.py

Currently these tools are supported:

* Rigid Tensor Registration
* Affine Tensor Registration
* Diffeomorphic Tensor Registration
* Combine affiine and diffeomorphic transforms
* Application of transform to tensor and scalar volumes
* Threshold and Binarize
* Adjusting the voxel space of tensor and scalar volumes
* Resampling tensor and scalar volumes
* Calculation of tensor metrics from tensor volume

Examples
--------
See the docstrings of the individual classes for examples.

"""
from __future__ import print_function, division, unicode_literals, \
    absolute_import

import os

from ... import logging
from ...utils.filemanip import fname_presuffix
from ..base import CommandLine
from nipype.interfaces.fsl.base import Info
import warnings

LOGGER = logging.getLogger('nipype.interface')


class DTITKRenameMixin(object):
    def __init__(self, *args, **kwargs):
        classes = [cls.__name__ for cls in self.__class__.mro()]
        dep_name = classes[0]
        rename_idx = classes.index('DTITKRenameMixin')
        new_name = classes[rename_idx + 1]
        warnings.warn('The {} interface has been renamed to {}\n'
                      'Please see the documentation for DTI-TK '
                      'interfaces, as some inputs have been '
                      'added or renamed for clarity.'
                      ''.format(dep_name, new_name),
                      DeprecationWarning)
        super(DTITKRenameMixin, self).__init__(*args, **kwargs)


class CommandLineDtitk(CommandLine):

    def _gen_fname(self, basename, cwd=None, suffix=None, change_ext=True,
                   ext=None):
        """Generate a filename based on the given parameters.

        The filename will take the form: cwd/basename<suffix><ext>.
        If change_ext is True, it will use the extentions specified in
        <instance>intputs.output_type.

        Parameters
        ----------
        basename : str
            Filename to base the new filename on.
        cwd : str
            Path to prefix to the new filename. (default is os.getcwd())
        suffix : str
            Suffix to add to the `basename`.  (defaults is '' )
        change_ext : bool
            Flag to change the filename extension to the FSL output type.
            (default True)

        Returns
        -------
        fname : str
            New filename based on given parameters.

        """

        if basename == '':
            msg = 'Unable to generate filename for command %s. ' % self.cmd
            msg += 'basename is not set!'
            raise ValueError(msg)
        if cwd is None:
            cwd = os.getcwd()
        if ext is None:
            ext = Info.output_type_to_ext(self.inputs.output_type)
        if change_ext:
            if suffix:
                suffix = ''.join((suffix, ext))
            else:
                suffix = ext
        if suffix is None:
            suffix = ''
        fname = fname_presuffix(basename, suffix=suffix,
                                use_ext=False, newpath=cwd)
        return fname



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import CenterMass


def test_CenterMass_inputs():
    input_map = dict(
        all_rois=dict(argstr='-all_rois', ),
        args=dict(argstr='%s', ),
        automask=dict(argstr='-automask', ),
        cm_file=dict(
            argstr='> %s',
            extensions=None,
            hash_files=False,
            keep_extension=False,
            name_source='in_file',
            name_template='%s_cm.out',
            position=-1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            copyfile=True,
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        local_ijk=dict(argstr='-local_ijk', ),
        mask_file=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        roi_vals=dict(argstr='-roi_vals %s', ),
        set_cm=dict(argstr='-set %f %f %f', ),
    )
    inputs = CenterMass.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_CenterMass_outputs():
    output_map = dict(
        cm=dict(),
        cm_file=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
    )
    outputs = CenterMass.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..developer import JistLaminarProfileSampling


def test_JistLaminarProfileSampling_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inCortex=dict(
            argstr='--inCortex %s',
            extensions=None,
        ),
        inIntensity=dict(
            argstr='--inIntensity %s',
            extensions=None,
        ),
        inProfile=dict(
            argstr='--inProfile %s',
            extensions=None,
        ),
        null=dict(argstr='--null %s', ),
        outProfile2=dict(
            argstr='--outProfile2 %s',
            hash_files=False,
        ),
        outProfilemapped=dict(
            argstr='--outProfilemapped %s',
            hash_files=False,
        ),
        xDefaultMem=dict(argstr='-xDefaultMem %d', ),
        xMaxProcess=dict(
            argstr='-xMaxProcess %d',
            usedefault=True,
        ),
        xPrefExt=dict(argstr='--xPrefExt %s', ),
    )
    inputs = JistLaminarProfileSampling.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_JistLaminarProfileSampling_outputs():
    output_map = dict(
        outProfile2=dict(extensions=None, ),
        outProfilemapped=dict(extensions=None, ),
    )
    outputs = JistLaminarProfileSampling.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Normalize12


def test_Normalize12_inputs():
    input_map = dict(
        affine_regularization_type=dict(field='eoptions.affreg', ),
        apply_to_files=dict(
            copyfile=True,
            field='subj.resample',
        ),
        bias_fwhm=dict(field='eoptions.biasfwhm', ),
        bias_regularization=dict(field='eoptions.biasreg', ),
        deformation_file=dict(
            copyfile=False,
            extensions=['.hdr', '.img', '.img.gz', '.nii'],
            field='subj.def',
            mandatory=True,
            xor=['image_to_align', 'tpm'],
        ),
        image_to_align=dict(
            copyfile=True,
            extensions=['.hdr', '.img', '.img.gz', '.nii'],
            field='subj.vol',
            mandatory=True,
            xor=['deformation_file'],
        ),
        jobtype=dict(usedefault=True, ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        out_prefix=dict(
            field='woptions.prefix',
            usedefault=True,
        ),
        paths=dict(),
        sampling_distance=dict(field='eoptions.samp', ),
        smoothness=dict(field='eoptions.fwhm', ),
        tpm=dict(
            copyfile=False,
            extensions=None,
            field='eoptions.tpm',
            xor=['deformation_file'],
        ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
        warping_regularization=dict(field='eoptions.reg', ),
        write_bounding_box=dict(field='woptions.bb', ),
        write_interp=dict(field='woptions.interp', ),
        write_voxel_sizes=dict(field='woptions.vox', ),
    )
    inputs = Normalize12.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Normalize12_outputs():
    output_map = dict(
        deformation_field=dict(),
        normalized_files=dict(),
        normalized_image=dict(),
    )
    outputs = Normalize12.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ..base import (CommandLine, CommandLineInputSpec, SEMLikeCommandLine,
                    TraitedSpec, File, Directory, traits, isdefined,
                    InputMultiPath, OutputMultiPath)


class JistLaminarVolumetricLayeringInputSpec(CommandLineInputSpec):
    inInner = File(
        desc="Inner Distance Image (GM/WM boundary)",
        exists=True,
        argstr="--inInner %s")
    inOuter = File(
        desc="Outer Distance Image (CSF/GM boundary)",
        exists=True,
        argstr="--inOuter %s")
    inNumber = traits.Int(desc="Number of layers", argstr="--inNumber %d")
    inMax = traits.Int(
        desc="Max iterations for narrow band evolution", argstr="--inMax %d")
    inMin = traits.Float(
        desc="Min change ratio for narrow band evolution", argstr="--inMin %f")
    inLayering = traits.Enum(
        "distance-preserving",
        "volume-preserving",
        desc="Layering method",
        argstr="--inLayering %s")
    inLayering2 = traits.Enum(
        "outward",
        "inward",
        desc="Layering direction",
        argstr="--inLayering2 %s")
    incurvature = traits.Int(
        desc="curvature approximation scale (voxels)",
        argstr="--incurvature %d")
    inratio = traits.Float(
        desc="ratio smoothing kernel size (voxels)", argstr="--inratio %f")
    inpresmooth = traits.Enum(
        "true",
        "false",
        desc="pre-smooth cortical surfaces",
        argstr="--inpresmooth %s")
    inTopology = traits.Enum(
        "26/6",
        "6/26",
        "18/6",
        "6/18",
        "6/6",
        "wcs",
        "wco",
        "no",
        desc="Topology",
        argstr="--inTopology %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outContinuous = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Continuous depth measurement",
        argstr="--outContinuous %s")
    outDiscrete = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Discrete sampled layers",
        argstr="--outDiscrete %s")
    outLayer = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Layer boundary surfaces",
        argstr="--outLayer %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistLaminarVolumetricLayeringOutputSpec(TraitedSpec):
    outContinuous = File(desc="Continuous depth measurement", exists=True)
    outDiscrete = File(desc="Discrete sampled layers", exists=True)
    outLayer = File(desc="Layer boundary surfaces", exists=True)


class JistLaminarVolumetricLayering(SEMLikeCommandLine):
    """title: Volumetric Layering

category: Developer Tools

description: Builds a continuous layering of the cortex following distance-preserving or volume-preserving models of cortical folding.
Waehnert MD, Dinse J, Weiss M, Streicher MN, Waehnert P, Geyer S, Turner R, Bazin PL, Anatomically motivated modeling of cortical laminae, Neuroimage, 2013.

version: 3.0.RC

contributor: Miriam Waehnert (waehnert@cbs.mpg.de) http://www.cbs.mpg.de/

"""

    input_spec = JistLaminarVolumetricLayeringInputSpec
    output_spec = JistLaminarVolumetricLayeringOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.laminar.JistLaminarVolumetricLayering "
    _outputs_filenames = {
        'outContinuous': 'outContinuous.nii',
        'outLayer': 'outLayer.nii',
        'outDiscrete': 'outDiscrete.nii'
    }
    _redirect_x = True


class JistBrainMgdmSegmentationInputSpec(CommandLineInputSpec):
    inMP2RAGE = File(
        desc="MP2RAGE T1 Map Image", exists=True, argstr="--inMP2RAGE %s")
    inMP2RAGE2 = File(
        desc="MP2RAGE T1-weighted Image",
        exists=True,
        argstr="--inMP2RAGE2 %s")
    inPV = File(desc="PV / Dura Image", exists=True, argstr="--inPV %s")
    inMPRAGE = File(
        desc="MPRAGE T1-weighted Image", exists=True, argstr="--inMPRAGE %s")
    inFLAIR = File(desc="FLAIR Image", exists=True, argstr="--inFLAIR %s")
    inAtlas = File(desc="Atlas file", exists=True, argstr="--inAtlas %s")
    inData = traits.Float(desc="Data weight", argstr="--inData %f")
    inCurvature = traits.Float(
        desc="Curvature weight", argstr="--inCurvature %f")
    inPosterior = traits.Float(
        desc="Posterior scale (mm)", argstr="--inPosterior %f")
    inMax = traits.Int(desc="Max iterations", argstr="--inMax %d")
    inMin = traits.Float(desc="Min change", argstr="--inMin %f")
    inSteps = traits.Int(desc="Steps", argstr="--inSteps %d")
    inTopology = traits.Enum(
        "26/6",
        "6/26",
        "18/6",
        "6/18",
        "6/6",
        "wcs",
        "wco",
        "no",
        desc="Topology",
        argstr="--inTopology %s")
    inCompute = traits.Enum(
        "true", "false", desc="Compute posteriors", argstr="--inCompute %s")
    inAdjust = traits.Enum(
        "true",
        "false",
        desc="Adjust intensity priors",
        argstr="--inAdjust %s")
    inOutput = traits.Enum(
        "segmentation",
        "memberships",
        desc="Output images",
        argstr="--inOutput %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outSegmented = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Segmented Brain Image",
        argstr="--outSegmented %s")
    outLevelset = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Levelset Boundary Image",
        argstr="--outLevelset %s")
    outPosterior2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Posterior Maximum Memberships (4D)",
        argstr="--outPosterior2 %s")
    outPosterior3 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Posterior Maximum Labels (4D)",
        argstr="--outPosterior3 %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistBrainMgdmSegmentationOutputSpec(TraitedSpec):
    outSegmented = File(desc="Segmented Brain Image", exists=True)
    outLevelset = File(desc="Levelset Boundary Image", exists=True)
    outPosterior2 = File(
        desc="Posterior Maximum Memberships (4D)", exists=True)
    outPosterior3 = File(desc="Posterior Maximum Labels (4D)", exists=True)


class JistBrainMgdmSegmentation(SEMLikeCommandLine):
    """title: MGDM Whole Brain Segmentation

category: Developer Tools

description: Estimate brain structures from an atlas for a MRI dataset (multiple input combinations are possible).

version: 2.0.RC

"""

    input_spec = JistBrainMgdmSegmentationInputSpec
    output_spec = JistBrainMgdmSegmentationOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.brain.JistBrainMgdmSegmentation "
    _outputs_filenames = {
        'outSegmented': 'outSegmented.nii',
        'outPosterior2': 'outPosterior2.nii',
        'outPosterior3': 'outPosterior3.nii',
        'outLevelset': 'outLevelset.nii'
    }
    _redirect_x = True


class JistLaminarProfileGeometryInputSpec(CommandLineInputSpec):
    inProfile = File(
        desc="Profile Surface Image", exists=True, argstr="--inProfile %s")
    incomputed = traits.Enum(
        "thickness",
        "curvedness",
        "shape_index",
        "mean_curvature",
        "gauss_curvature",
        "profile_length",
        "profile_curvature",
        "profile_torsion",
        desc="computed measure",
        argstr="--incomputed %s")
    inregularization = traits.Enum(
        "none",
        "Gaussian",
        desc="regularization",
        argstr="--inregularization %s")
    insmoothing = traits.Float(
        desc="smoothing parameter", argstr="--insmoothing %f")
    inoutside = traits.Float(
        desc="outside extension (mm)", argstr="--inoutside %f")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outResult = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Result",
        argstr="--outResult %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistLaminarProfileGeometryOutputSpec(TraitedSpec):
    outResult = File(desc="Result", exists=True)


class JistLaminarProfileGeometry(SEMLikeCommandLine):
    """title: Profile Geometry

category: Developer Tools

description: Compute various geometric quantities for a cortical layers.

version: 3.0.RC

"""

    input_spec = JistLaminarProfileGeometryInputSpec
    output_spec = JistLaminarProfileGeometryOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.laminar.JistLaminarProfileGeometry "
    _outputs_filenames = {'outResult': 'outResult.nii'}
    _redirect_x = True


class JistLaminarProfileCalculatorInputSpec(CommandLineInputSpec):
    inIntensity = File(
        desc="Intensity Profile Image", exists=True, argstr="--inIntensity %s")
    inMask = File(
        desc="Mask Image (opt, 3D or 4D)", exists=True, argstr="--inMask %s")
    incomputed = traits.Enum(
        "mean",
        "stdev",
        "skewness",
        "kurtosis",
        desc="computed statistic",
        argstr="--incomputed %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outResult = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Result",
        argstr="--outResult %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistLaminarProfileCalculatorOutputSpec(TraitedSpec):
    outResult = File(desc="Result", exists=True)


class JistLaminarProfileCalculator(SEMLikeCommandLine):
    """title: Profile Calculator

category: Developer Tools

description: Compute various moments for intensities mapped along a cortical profile.

version: 3.0.RC

"""

    input_spec = JistLaminarProfileCalculatorInputSpec
    output_spec = JistLaminarProfileCalculatorOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.laminar.JistLaminarProfileCalculator "
    _outputs_filenames = {'outResult': 'outResult.nii'}
    _redirect_x = True


class MedicAlgorithmN3InputSpec(CommandLineInputSpec):
    inInput = File(desc="Input Volume", exists=True, argstr="--inInput %s")
    inSignal = traits.Float(
        desc=
        "Default = min + 1, Values at less than threshold are treated as part of the background",
        argstr="--inSignal %f")
    inMaximum = traits.Int(
        desc="Maximum number of Iterations", argstr="--inMaximum %d")
    inEnd = traits.Float(
        desc=
        "Usually 0.01-0.00001, The measure used to terminate the iterations is the coefficient of variation of change in field estimates between successive iterations.",
        argstr="--inEnd %f")
    inField = traits.Float(
        desc=
        "Characteristic distance over which the field varies. The distance between adjacent knots in bspline fitting with at least 4 knots going in every dimension. The default in the dialog is one third the distance (resolution * extents) of the smallest dimension.",
        argstr="--inField %f")
    inSubsample = traits.Float(
        desc=
        "Usually between 1-32, The factor by which the data is subsampled to a lower resolution in estimating the slowly varying non-uniformity field. Reduce sampling in the finest sampling direction by the shrink factor.",
        argstr="--inSubsample %f")
    inKernel = traits.Float(
        desc=
        "Usually between 0.05-0.50, Width of deconvolution kernel used to sharpen the histogram. Larger values give faster convergence while smaller values give greater accuracy.",
        argstr="--inKernel %f")
    inWeiner = traits.Float(
        desc="Usually between 0.0-1.0", argstr="--inWeiner %f")
    inAutomatic = traits.Enum(
        "true",
        "false",
        desc=
        "If true determines the threshold by histogram analysis. If true a VOI cannot be used and the input threshold is ignored.",
        argstr="--inAutomatic %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outInhomogeneity = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Inhomogeneity Corrected Volume",
        argstr="--outInhomogeneity %s")
    outInhomogeneity2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Inhomogeneity Field",
        argstr="--outInhomogeneity2 %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class MedicAlgorithmN3OutputSpec(TraitedSpec):
    outInhomogeneity = File(desc="Inhomogeneity Corrected Volume", exists=True)
    outInhomogeneity2 = File(desc="Inhomogeneity Field", exists=True)


class MedicAlgorithmN3(SEMLikeCommandLine):
    """title: N3 Correction

category: Developer Tools

description: Non-parametric Intensity Non-uniformity Correction, N3, originally by J.G. Sled.

version: 1.8.R

"""

    input_spec = MedicAlgorithmN3InputSpec
    output_spec = MedicAlgorithmN3OutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.classification.MedicAlgorithmN3 "
    _outputs_filenames = {
        'outInhomogeneity2': 'outInhomogeneity2.nii',
        'outInhomogeneity': 'outInhomogeneity.nii'
    }
    _redirect_x = True


class JistLaminarROIAveragingInputSpec(CommandLineInputSpec):
    inIntensity = File(
        desc="Intensity Profile Image", exists=True, argstr="--inIntensity %s")
    inROI = File(desc="ROI Mask", exists=True, argstr="--inROI %s")
    inROI2 = traits.Str(desc="ROI Name", argstr="--inROI2 %s")
    inMask = File(
        desc="Mask Image (opt, 3D or 4D)", exists=True, argstr="--inMask %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outROI3 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="ROI Average",
        argstr="--outROI3 %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistLaminarROIAveragingOutputSpec(TraitedSpec):
    outROI3 = File(desc="ROI Average", exists=True)


class JistLaminarROIAveraging(SEMLikeCommandLine):
    """title: Profile ROI Averaging

category: Developer Tools

description: Compute an average profile over a given ROI.

version: 3.0.RC

"""

    input_spec = JistLaminarROIAveragingInputSpec
    output_spec = JistLaminarROIAveragingOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.laminar.JistLaminarROIAveraging "
    _outputs_filenames = {'outROI3': 'outROI3'}
    _redirect_x = True


class MedicAlgorithmLesionToadsInputSpec(CommandLineInputSpec):
    inT1_MPRAGE = File(
        desc="T1_MPRAGE Image", exists=True, argstr="--inT1_MPRAGE %s")
    inT1_SPGR = File(
        desc="T1_SPGR Image", exists=True, argstr="--inT1_SPGR %s")
    inFLAIR = File(desc="FLAIR Image", exists=True, argstr="--inFLAIR %s")
    inAtlas = traits.Enum(
        "With Lesion", "No Lesion", desc="Atlas to Use", argstr="--inAtlas %s")
    inOutput = traits.Enum(
        "hard segmentation",
        "hard segmentation+memberships",
        "cruise inputs",
        "dura removal inputs",
        desc="Output images",
        argstr="--inOutput %s")
    inOutput2 = traits.Enum(
        "true",
        "false",
        desc=
        "Output the hard classification using maximum membership (not neceesarily topologically correct)",
        argstr="--inOutput2 %s")
    inCorrect = traits.Enum(
        "true",
        "false",
        desc="Correct MR field inhomogeneity.",
        argstr="--inCorrect %s")
    inOutput3 = traits.Enum(
        "true",
        "false",
        desc="Output the estimated inhomogeneity field",
        argstr="--inOutput3 %s")
    inAtlas2 = File(
        desc="Atlas File - With Lesions", exists=True, argstr="--inAtlas2 %s")
    inAtlas3 = File(
        desc="Atlas File - No Lesion - T1 and FLAIR",
        exists=True,
        argstr="--inAtlas3 %s")
    inAtlas4 = File(
        desc="Atlas File - No Lesion - T1 Only",
        exists=True,
        argstr="--inAtlas4 %s")
    inMaximum = traits.Int(
        desc=
        "Maximum distance from the interventricular WM boundary to downweight the lesion membership to avoid false postives",
        argstr="--inMaximum %d")
    inMaximum2 = traits.Int(
        desc="Maximum Ventircle Distance", argstr="--inMaximum2 %d")
    inMaximum3 = traits.Int(
        desc="Maximum InterVentricular Distance", argstr="--inMaximum3 %d")
    inInclude = traits.Enum(
        "true",
        "false",
        desc="Include lesion in WM class in hard classification",
        argstr="--inInclude %s")
    inAtlas5 = traits.Float(
        desc="Controls the effect of the statistical atlas on the segmentation",
        argstr="--inAtlas5 %f")
    inSmooting = traits.Float(
        desc="Controls the effect of neighberhood voxels on the membership",
        argstr="--inSmooting %f")
    inMaximum4 = traits.Float(
        desc=
        "Maximum amount of relative change in the energy function considered as the convergence criteria",
        argstr="--inMaximum4 %f")
    inMaximum5 = traits.Int(
        desc="Maximum iterations", argstr="--inMaximum5 %d")
    inAtlas6 = traits.Enum(
        "rigid",
        "multi_fully_affine",
        desc="Atlas alignment",
        argstr="--inAtlas6 %s")
    inConnectivity = traits.Enum(
        "(26,6)",
        "(6,26)",
        "(6,18)",
        "(18,6)",
        desc="Connectivity (foreground,background)",
        argstr="--inConnectivity %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outHard = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Hard segmentation",
        argstr="--outHard %s")
    outHard2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Hard segmentationfrom memberships",
        argstr="--outHard2 %s")
    outInhomogeneity = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Inhomogeneity Field",
        argstr="--outInhomogeneity %s")
    outMembership = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Membership Functions",
        argstr="--outMembership %s")
    outLesion = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Lesion Segmentation",
        argstr="--outLesion %s")
    outSulcal = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Sulcal CSF Membership",
        argstr="--outSulcal %s")
    outCortical = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Cortical GM Membership",
        argstr="--outCortical %s")
    outFilled = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Filled WM Membership",
        argstr="--outFilled %s")
    outWM = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="WM Mask",
        argstr="--outWM %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class MedicAlgorithmLesionToadsOutputSpec(TraitedSpec):
    outHard = File(desc="Hard segmentation", exists=True)
    outHard2 = File(desc="Hard segmentationfrom memberships", exists=True)
    outInhomogeneity = File(desc="Inhomogeneity Field", exists=True)
    outMembership = File(desc="Membership Functions", exists=True)
    outLesion = File(desc="Lesion Segmentation", exists=True)
    outSulcal = File(desc="Sulcal CSF Membership", exists=True)
    outCortical = File(desc="Cortical GM Membership", exists=True)
    outFilled = File(desc="Filled WM Membership", exists=True)
    outWM = File(desc="WM Mask", exists=True)


class MedicAlgorithmLesionToads(SEMLikeCommandLine):
    """title: Lesion TOADS

category: Developer Tools

description: Algorithm for simulataneous brain structures and MS lesion segmentation of MS Brains. The brain segmentation is topologically consistent and the algorithm can use multiple MR sequences as input data.
N. Shiee, P.-L. Bazin, A.Z. Ozturk, P.A. Calabresi, D.S. Reich, D.L. Pham, "A Topology-Preserving Approach to the Segmentation of Brain Images with Multiple Sclerosis", NeuroImage, vol. 49, no. 2, pp. 1524-1535, 2010.

version: 1.9.R

contributor: Navid Shiee (navid.shiee@nih.gov) http://iacl.ece.jhu.edu/~nshiee/

"""

    input_spec = MedicAlgorithmLesionToadsInputSpec
    output_spec = MedicAlgorithmLesionToadsOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.classification.MedicAlgorithmLesionToads "
    _outputs_filenames = {
        'outWM': 'outWM.nii',
        'outHard': 'outHard.nii',
        'outFilled': 'outFilled.nii',
        'outMembership': 'outMembership.nii',
        'outInhomogeneity': 'outInhomogeneity.nii',
        'outCortical': 'outCortical.nii',
        'outHard2': 'outHard2.nii',
        'outLesion': 'outLesion.nii',
        'outSulcal': 'outSulcal.nii'
    }
    _redirect_x = True


class JistBrainMp2rageSkullStrippingInputSpec(CommandLineInputSpec):
    inSecond = File(
        desc="Second inversion (Inv2) Image",
        exists=True,
        argstr="--inSecond %s")
    inT1 = File(
        desc="T1 Map (T1_Images) Image (opt)", exists=True, argstr="--inT1 %s")
    inT1weighted = File(
        desc="T1-weighted (UNI) Image (opt)",
        exists=True,
        argstr="--inT1weighted %s")
    inFilter = File(
        desc="Filter Image (opt)", exists=True, argstr="--inFilter %s")
    inSkip = traits.Enum(
        "true", "false", desc="Skip zero values", argstr="--inSkip %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outBrain = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Brain Mask Image",
        argstr="--outBrain %s")
    outMasked = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Masked T1 Map Image",
        argstr="--outMasked %s")
    outMasked2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Masked T1-weighted Image",
        argstr="--outMasked2 %s")
    outMasked3 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Masked Filter Image",
        argstr="--outMasked3 %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistBrainMp2rageSkullStrippingOutputSpec(TraitedSpec):
    outBrain = File(desc="Brain Mask Image", exists=True)
    outMasked = File(desc="Masked T1 Map Image", exists=True)
    outMasked2 = File(desc="Masked T1-weighted Image", exists=True)
    outMasked3 = File(desc="Masked Filter Image", exists=True)


class JistBrainMp2rageSkullStripping(SEMLikeCommandLine):
    """title: MP2RAGE Skull Stripping

category: Developer Tools

description: Estimate a brain mask for a MP2RAGE dataset. At least a T1-weighted or a T1 map image is required.

version: 3.0.RC

"""

    input_spec = JistBrainMp2rageSkullStrippingInputSpec
    output_spec = JistBrainMp2rageSkullStrippingOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.brain.JistBrainMp2rageSkullStripping "
    _outputs_filenames = {
        'outBrain': 'outBrain.nii',
        'outMasked3': 'outMasked3.nii',
        'outMasked2': 'outMasked2.nii',
        'outMasked': 'outMasked.nii'
    }
    _redirect_x = True


class JistCortexSurfaceMeshInflationInputSpec(CommandLineInputSpec):
    inLevelset = File(
        desc="Levelset Image", exists=True, argstr="--inLevelset %s")
    inSOR = traits.Float(desc="SOR Parameter", argstr="--inSOR %f")
    inMean = traits.Float(
        desc="Mean Curvature Threshold", argstr="--inMean %f")
    inStep = traits.Int(desc="Step Size", argstr="--inStep %d")
    inMax = traits.Int(desc="Max Iterations", argstr="--inMax %d")
    inLorentzian = traits.Enum(
        "true", "false", desc="Lorentzian Norm", argstr="--inLorentzian %s")
    inTopology = traits.Enum(
        "26/6",
        "6/26",
        "18/6",
        "6/18",
        "6/6",
        "wcs",
        "wco",
        "no",
        desc="Topology",
        argstr="--inTopology %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outOriginal = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Original Surface",
        argstr="--outOriginal %s")
    outInflated = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Inflated Surface",
        argstr="--outInflated %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistCortexSurfaceMeshInflationOutputSpec(TraitedSpec):
    outOriginal = File(desc="Original Surface", exists=True)
    outInflated = File(desc="Inflated Surface", exists=True)


class JistCortexSurfaceMeshInflation(SEMLikeCommandLine):
    """title: Surface Mesh Inflation

category: Developer Tools

description: Inflates a cortical surface mesh.
D. Tosun, M. E. Rettmann, X. Han, X. Tao, C. Xu, S. M. Resnick, D. Pham, and J. L. Prince, Cortical Surface Segmentation and Mapping, NeuroImage, vol. 23, pp. S108--S118, 2004.

version: 3.0.RC

contributor: Duygu Tosun

"""

    input_spec = JistCortexSurfaceMeshInflationInputSpec
    output_spec = JistCortexSurfaceMeshInflationOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.cortex.JistCortexSurfaceMeshInflation "
    _outputs_filenames = {
        'outOriginal': 'outOriginal',
        'outInflated': 'outInflated'
    }
    _redirect_x = True


class RandomVolInputSpec(CommandLineInputSpec):
    inSize = traits.Int(
        desc="Size of Volume in X direction", argstr="--inSize %d")
    inSize2 = traits.Int(
        desc="Size of Volume in Y direction", argstr="--inSize2 %d")
    inSize3 = traits.Int(
        desc="Size of Volume in Z direction", argstr="--inSize3 %d")
    inSize4 = traits.Int(
        desc="Size of Volume in t direction", argstr="--inSize4 %d")
    inStandard = traits.Int(
        desc="Standard Deviation for Normal Distribution",
        argstr="--inStandard %d")
    inLambda = traits.Float(
        desc="Lambda Value for Exponential Distribution",
        argstr="--inLambda %f")
    inMaximum = traits.Int(desc="Maximum Value", argstr="--inMaximum %d")
    inMinimum = traits.Int(desc="Minimum Value", argstr="--inMinimum %d")
    inField = traits.Enum(
        "Uniform",
        "Normal",
        "Exponential",
        desc="Field",
        argstr="--inField %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outRand1 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Rand1",
        argstr="--outRand1 %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class RandomVolOutputSpec(TraitedSpec):
    outRand1 = File(desc="Rand1", exists=True)


class RandomVol(SEMLikeCommandLine):
    """title: Random Volume Generator

category: Developer Tools

description: Generate a random scalar volume.

version: 1.12.RC

documentation-url: http://www.nitrc.org/projects/jist/

"""

    input_spec = RandomVolInputSpec
    output_spec = RandomVolOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.bme.smile.demo.RandomVol "
    _outputs_filenames = {'outRand1': 'outRand1.nii'}
    _redirect_x = True


class MedicAlgorithmImageCalculatorInputSpec(CommandLineInputSpec):
    inVolume = File(desc="Volume 1", exists=True, argstr="--inVolume %s")
    inVolume2 = File(desc="Volume 2", exists=True, argstr="--inVolume2 %s")
    inOperation = traits.Enum(
        "Add",
        "Subtract",
        "Multiply",
        "Divide",
        "Min",
        "Max",
        desc="Operation",
        argstr="--inOperation %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outResult = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Result Volume",
        argstr="--outResult %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class MedicAlgorithmImageCalculatorOutputSpec(TraitedSpec):
    outResult = File(desc="Result Volume", exists=True)


class MedicAlgorithmImageCalculator(SEMLikeCommandLine):
    """title: Image Calculator

category: Developer Tools

description: Perform simple image calculator operations on two images. The operations include 'Add', 'Subtract', 'Multiply', and 'Divide'

version: 1.10.RC

documentation-url: http://www.iacl.ece.jhu.edu/

"""

    input_spec = MedicAlgorithmImageCalculatorInputSpec
    output_spec = MedicAlgorithmImageCalculatorOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.utilities.math.MedicAlgorithmImageCalculator "
    _outputs_filenames = {'outResult': 'outResult.nii'}
    _redirect_x = True


class JistBrainMp2rageDuraEstimationInputSpec(CommandLineInputSpec):
    inSecond = File(
        desc="Second inversion (Inv2) Image",
        exists=True,
        argstr="--inSecond %s")
    inSkull = File(
        desc="Skull Stripping Mask", exists=True, argstr="--inSkull %s")
    inDistance = traits.Float(
        desc="Distance to background (mm)", argstr="--inDistance %f")
    inoutput = traits.Enum(
        "dura_region",
        "boundary",
        "dura_prior",
        "bg_prior",
        "intens_prior",
        desc=
        "Outputs an estimate of the dura / CSF boundary or an estimate of the entire dura region.",
        argstr="--inoutput %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outDura = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Dura Image",
        argstr="--outDura %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistBrainMp2rageDuraEstimationOutputSpec(TraitedSpec):
    outDura = File(desc="Dura Image", exists=True)


class JistBrainMp2rageDuraEstimation(SEMLikeCommandLine):
    """title: MP2RAGE Dura Estimation

category: Developer Tools

description: Filters a MP2RAGE brain image to obtain a probability map of dura matter.

version: 3.0.RC

"""

    input_spec = JistBrainMp2rageDuraEstimationInputSpec
    output_spec = JistBrainMp2rageDuraEstimationOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.brain.JistBrainMp2rageDuraEstimation "
    _outputs_filenames = {'outDura': 'outDura.nii'}
    _redirect_x = True


class JistLaminarProfileSamplingInputSpec(CommandLineInputSpec):
    inProfile = File(
        desc="Profile Surface Image", exists=True, argstr="--inProfile %s")
    inIntensity = File(
        desc="Intensity Image", exists=True, argstr="--inIntensity %s")
    inCortex = File(
        desc="Cortex Mask (opt)", exists=True, argstr="--inCortex %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outProfilemapped = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Profile-mapped Intensity Image",
        argstr="--outProfilemapped %s")
    outProfile2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Profile 4D Mask",
        argstr="--outProfile2 %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistLaminarProfileSamplingOutputSpec(TraitedSpec):
    outProfilemapped = File(desc="Profile-mapped Intensity Image", exists=True)
    outProfile2 = File(desc="Profile 4D Mask", exists=True)


class JistLaminarProfileSampling(SEMLikeCommandLine):
    """title: Profile Sampling

category: Developer Tools

description: Sample some intensity image along a cortical profile across layer surfaces.

version: 3.0.RC

"""

    input_spec = JistLaminarProfileSamplingInputSpec
    output_spec = JistLaminarProfileSamplingOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.laminar.JistLaminarProfileSampling "
    _outputs_filenames = {
        'outProfile2': 'outProfile2.nii',
        'outProfilemapped': 'outProfilemapped.nii'
    }
    _redirect_x = True


class MedicAlgorithmMipavReorientInputSpec(CommandLineInputSpec):
    inSource = InputMultiPath(
        File, desc="Source", sep=";", argstr="--inSource %s")
    inTemplate = File(desc="Template", exists=True, argstr="--inTemplate %s")
    inNew = traits.Enum(
        "Dicom axial",
        "Dicom coronal",
        "Dicom sagittal",
        "User defined",
        desc="New image orientation",
        argstr="--inNew %s")
    inUser = traits.Enum(
        "Unknown",
        "Patient Right to Left",
        "Patient Left to Right",
        "Patient Posterior to Anterior",
        "Patient Anterior to Posterior",
        "Patient Inferior to Superior",
        "Patient Superior to Inferior",
        desc="User defined X-axis orientation (image left to right)",
        argstr="--inUser %s")
    inUser2 = traits.Enum(
        "Unknown",
        "Patient Right to Left",
        "Patient Left to Right",
        "Patient Posterior to Anterior",
        "Patient Anterior to Posterior",
        "Patient Inferior to Superior",
        "Patient Superior to Inferior",
        desc="User defined Y-axis orientation (image top to bottom)",
        argstr="--inUser2 %s")
    inUser3 = traits.Enum(
        "Unknown",
        "Patient Right to Left",
        "Patient Left to Right",
        "Patient Posterior to Anterior",
        "Patient Anterior to Posterior",
        "Patient Inferior to Superior",
        "Patient Superior to Inferior",
        desc="User defined Z-axis orientation (into the screen)",
        argstr="--inUser3 %s")
    inUser4 = traits.Enum(
        "Axial",
        "Coronal",
        "Sagittal",
        "Unknown",
        desc="User defined Image Orientation",
        argstr="--inUser4 %s")
    inInterpolation = traits.Enum(
        "Nearest Neighbor",
        "Trilinear",
        "Bspline 3rd order",
        "Bspline 4th order",
        "Cubic Lagrangian",
        "Quintic Lagrangian",
        "Heptic Lagrangian",
        "Windowed Sinc",
        desc="Interpolation",
        argstr="--inInterpolation %s")
    inResolution = traits.Enum(
        "Unchanged",
        "Finest cubic",
        "Coarsest cubic",
        "Same as template",
        desc="Resolution",
        argstr="--inResolution %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outReoriented = InputMultiPath(
        File, desc="Reoriented Volume", sep=";", argstr="--outReoriented %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class MedicAlgorithmMipavReorientOutputSpec(TraitedSpec):
    pass


class MedicAlgorithmMipavReorient(SEMLikeCommandLine):
    """title: Reorient Volume

category: Developer Tools

description: Reorient a volume to a particular anatomical orientation.

version: .alpha

"""

    input_spec = MedicAlgorithmMipavReorientInputSpec
    output_spec = MedicAlgorithmMipavReorientOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.utilities.volume.MedicAlgorithmMipavReorient "
    _outputs_filenames = {}
    _redirect_x = True


class MedicAlgorithmSPECTRE2010InputSpec(CommandLineInputSpec):
    inInput = File(
        desc="Input volume to be skullstripped.",
        exists=True,
        argstr="--inInput %s")
    inAtlas = File(
        desc=
        "SPECTRE atlas description file. A text file enumerating atlas files and landmarks.",
        exists=True,
        argstr="--inAtlas %s")
    inInitial = traits.Int(
        desc=
        "Erosion of the inital mask, which is based on the probability mask and the classification., The initial mask is ouput as the d0 volume at the conclusion of SPECTRE.",
        argstr="--inInitial %d")
    inImage = traits.Enum(
        "T1_SPGR",
        "T1_ALT",
        "T1_MPRAGE",
        "T2",
        "FLAIR",
        desc=
        "Set the image modality. MP-RAGE is recommended for most T1 sequence images.",
        argstr="--inImage %s")
    inOutput = traits.Enum(
        "true",
        "false",
        desc=
        "Determines if the output results are transformed back into the space of the original input image.",
        argstr="--inOutput %s")
    inFind = traits.Enum(
        "true", "false", desc="Find Midsaggital Plane", argstr="--inFind %s")
    inRun = traits.Enum(
        "true", "false", desc="Run Smooth Brain Mask", argstr="--inRun %s")
    inResample = traits.Enum(
        "true",
        "false",
        desc=
        "Determines if the data is resampled to be isotropic during the processing.",
        argstr="--inResample %s")
    inInitial2 = traits.Float(
        desc="Initial probability threshold", argstr="--inInitial2 %f")
    inMinimum = traits.Float(
        desc="Minimum probability threshold", argstr="--inMinimum %f")
    inMMC = traits.Int(
        desc=
        "The size of the dilation step within the Modified Morphological Closing.",
        argstr="--inMMC %d")
    inMMC2 = traits.Int(
        desc=
        "The size of the erosion step within the Modified Morphological Closing.",
        argstr="--inMMC2 %d")
    inInhomogeneity = traits.Enum(
        "true",
        "false",
        desc=
        "Set to false by default, this parameter will make FANTASM try to do inhomogeneity correction during it's iterative cycle.",
        argstr="--inInhomogeneity %s")
    inSmoothing = traits.Float(argstr="--inSmoothing %f")
    inBackground = traits.Float(argstr="--inBackground %f")
    inOutput2 = traits.Enum(
        "true", "false", desc="Output Plane?", argstr="--inOutput2 %s")
    inOutput3 = traits.Enum(
        "true", "false", desc="Output Split-Halves?", argstr="--inOutput3 %s")
    inOutput4 = traits.Enum(
        "true",
        "false",
        desc="Output Segmentation on Plane?",
        argstr="--inOutput4 %s")
    inDegrees = traits.Enum(
        "Rigid - 6",
        "Global rescale - 7",
        "Specific rescale - 9",
        "Affine - 12",
        desc="Degrees of freedom",
        argstr="--inDegrees %s")
    inCost = traits.Enum(
        "Correlation ratio",
        "Least squares",
        "Normalized cross correlation",
        "Normalized mutual information",
        desc="Cost function",
        argstr="--inCost %s")
    inRegistration = traits.Enum(
        "Trilinear",
        "Bspline 3rd order",
        "Bspline 4th order",
        "Cubic Lagrangian",
        "Quintic Lagrangian",
        "Heptic Lagrangian",
        "Windowed sinc",
        desc="Registration interpolation",
        argstr="--inRegistration %s")
    inOutput5 = traits.Enum(
        "Trilinear",
        "Bspline 3rd order",
        "Bspline 4th order",
        "Cubic Lagrangian",
        "Quintic Lagrangian",
        "Heptic Lagrangian",
        "Windowed sinc",
        "Nearest Neighbor",
        desc="Output interpolation",
        argstr="--inOutput5 %s")
    inApply = traits.Enum(
        "All", "X", "Y", "Z", desc="Apply rotation", argstr="--inApply %s")
    inMinimum2 = traits.Float(desc="Minimum angle", argstr="--inMinimum2 %f")
    inMaximum = traits.Float(desc="Maximum angle", argstr="--inMaximum %f")
    inCoarse = traits.Float(
        desc="Coarse angle increment", argstr="--inCoarse %f")
    inFine = traits.Float(desc="Fine angle increment", argstr="--inFine %f")
    inMultiple = traits.Int(
        desc="Multiple of tolerance to bracket the minimum",
        argstr="--inMultiple %d")
    inNumber = traits.Int(desc="Number of iterations", argstr="--inNumber %d")
    inNumber2 = traits.Int(
        desc="Number of minima from Level 8 to test at Level 4",
        argstr="--inNumber2 %d")
    inUse = traits.Enum(
        "true",
        "false",
        desc=
        "Use the max of the min resolutions of the two datasets when resampling",
        argstr="--inUse %s")
    inSubsample = traits.Enum(
        "true",
        "false",
        desc="Subsample image for speed",
        argstr="--inSubsample %s")
    inSkip = traits.Enum(
        "true",
        "false",
        desc="Skip multilevel search (Assume images are close to alignment)",
        argstr="--inSkip %s")
    inMultithreading = traits.Enum(
        "true",
        "false",
        desc=
        "Set to false by default, this parameter controls the multithreaded behavior of the linear registration.",
        argstr="--inMultithreading %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outOriginal = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "If Output in Original Space Flag is true then outputs the original input volume. Otherwise outputs the axialy reoriented input volume.",
        argstr="--outOriginal %s")
    outStripped = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Skullstripped result of the input volume with just the brain.",
        argstr="--outStripped %s")
    outMask = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Binary Mask of the skullstripped result with just the brain",
        argstr="--outMask %s")
    outPrior = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Probability prior from the atlas registrations",
        argstr="--outPrior %s")
    outFANTASM = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Tissue classification of of the whole input volume.",
        argstr="--outFANTASM %s")
    outd0 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Initial Brainmask",
        argstr="--outd0 %s")
    outMidsagittal = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Plane dividing the brain hemispheres",
        argstr="--outMidsagittal %s")
    outSplitHalves = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Skullstripped mask of the brain with the hemispheres divided.",
        argstr="--outSplitHalves %s")
    outSegmentation = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "2D image showing the tissue classification on the midsagittal plane",
        argstr="--outSegmentation %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class MedicAlgorithmSPECTRE2010OutputSpec(TraitedSpec):
    outOriginal = File(
        desc=
        "If Output in Original Space Flag is true then outputs the original input volume. Otherwise outputs the axialy reoriented input volume.",
        exists=True)
    outStripped = File(
        desc="Skullstripped result of the input volume with just the brain.",
        exists=True)
    outMask = File(
        desc="Binary Mask of the skullstripped result with just the brain",
        exists=True)
    outPrior = File(
        desc="Probability prior from the atlas registrations", exists=True)
    outFANTASM = File(
        desc="Tissue classification of of the whole input volume.",
        exists=True)
    outd0 = File(desc="Initial Brainmask", exists=True)
    outMidsagittal = File(
        desc="Plane dividing the brain hemispheres", exists=True)
    outSplitHalves = File(
        desc="Skullstripped mask of the brain with the hemispheres divided.",
        exists=True)
    outSegmentation = File(
        desc=
        "2D image showing the tissue classification on the midsagittal plane",
        exists=True)


class MedicAlgorithmSPECTRE2010(SEMLikeCommandLine):
    """title: SPECTRE 2010

category: Developer Tools

description: Simple Paradigm for Extra-Cranial Tissue REmoval

Algorithm Version: 1.6
GUI Version: 1.10

A. Carass, M.B. Wheeler, J. Cuzzocreo, P.-L. Bazin, S.S. Bassett, and J.L. Prince, 'A Joint Registration and Segmentation Approach to Skull Stripping', Fourth IEEE International Symposium on Biomedical Imaging (ISBI 2007), Arlington, VA, April 12-15, 2007.
A. Carass, J. Cuzzocreo, M.B. Wheeler, P.-L. Bazin, S.M. Resnick, and J.L. Prince, 'Simple paradigm for extra-cerebral tissue removal: Algorithm and analysis', NeuroImage 56(4):1982-1992, 2011.

version: 1.6.R

documentation-url: http://www.iacl.ece.jhu.edu/

contributor: Aaron Carass (aaron_carass@jhu.edu) http://www.iacl.ece.jhu.edu/
Hanlin Wan (hanlinwan@gmail.com)

"""

    input_spec = MedicAlgorithmSPECTRE2010InputSpec
    output_spec = MedicAlgorithmSPECTRE2010OutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.segmentation.skull_strip.MedicAlgorithmSPECTRE2010 "
    _outputs_filenames = {
        'outd0': 'outd0.nii',
        'outOriginal': 'outOriginal.nii',
        'outMask': 'outMask.nii',
        'outSplitHalves': 'outSplitHalves.nii',
        'outMidsagittal': 'outMidsagittal.nii',
        'outPrior': 'outPrior.nii',
        'outFANTASM': 'outFANTASM.nii',
        'outSegmentation': 'outSegmentation.nii',
        'outStripped': 'outStripped.nii'
    }
    _redirect_x = True


class JistBrainPartialVolumeFilterInputSpec(CommandLineInputSpec):
    inInput = File(desc="Input Image", exists=True, argstr="--inInput %s")
    inPV = traits.Enum(
        "bright",
        "dark",
        "both",
        desc=
        "Outputs the raw intensity values or a probability score for the partial volume regions.",
        argstr="--inPV %s")
    inoutput = traits.Enum(
        "probability", "intensity", desc="output", argstr="--inoutput %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outPartial = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Partial Volume Image",
        argstr="--outPartial %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistBrainPartialVolumeFilterOutputSpec(TraitedSpec):
    outPartial = File(desc="Partial Volume Image", exists=True)


class JistBrainPartialVolumeFilter(SEMLikeCommandLine):
    """title: Partial Volume Filter

category: Developer Tools

description: Filters an image for regions of partial voluming assuming a ridge-like model of intensity.

version: 2.0.RC

"""

    input_spec = JistBrainPartialVolumeFilterInputSpec
    output_spec = JistBrainPartialVolumeFilterOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.brain.JistBrainPartialVolumeFilter "
    _outputs_filenames = {'outPartial': 'outPartial.nii'}
    _redirect_x = True


class JistIntensityMp2rageMaskingInputSpec(CommandLineInputSpec):
    inSecond = File(
        desc="Second inversion (Inv2) Image",
        exists=True,
        argstr="--inSecond %s")
    inQuantitative = File(
        desc="Quantitative T1 Map (T1_Images) Image",
        exists=True,
        argstr="--inQuantitative %s")
    inT1weighted = File(
        desc="T1-weighted (UNI) Image",
        exists=True,
        argstr="--inT1weighted %s")
    inBackground = traits.Enum(
        "exponential",
        "half-normal",
        desc=
        "Model distribution for background noise (default is half-normal, exponential is more stringent).",
        argstr="--inBackground %s")
    inSkip = traits.Enum(
        "true", "false", desc="Skip zero values", argstr="--inSkip %s")
    inMasking = traits.Enum(
        "binary",
        "proba",
        desc=
        "Whether to use a binary threshold or a weighted average based on the probability.",
        argstr="--inMasking %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outSignal = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Signal Proba Image",
        argstr="--outSignal_Proba %s")
    outSignal2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Signal Mask Image",
        argstr="--outSignal_Mask %s")
    outMasked = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Masked T1 Map Image",
        argstr="--outMasked_T1_Map %s")
    outMasked2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Masked Iso Image",
        argstr="--outMasked_T1weighted %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class JistIntensityMp2rageMaskingOutputSpec(TraitedSpec):
    outSignal = File(desc="Signal Proba Image", exists=True)
    outSignal2 = File(desc="Signal Mask Image", exists=True)
    outMasked = File(desc="Masked T1 Map Image", exists=True)
    outMasked2 = File(desc="Masked Iso Image", exists=True)


class JistIntensityMp2rageMasking(SEMLikeCommandLine):
    """title: MP2RAGE Background Masking

category: Developer Tools

description: Estimate a background signal mask for a MP2RAGE dataset.

version: 3.0.RC

"""

    input_spec = JistIntensityMp2rageMaskingInputSpec
    output_spec = JistIntensityMp2rageMaskingOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.intensity.JistIntensityMp2rageMasking "
    _outputs_filenames = {
        'outSignal2': 'outSignal2.nii',
        'outSignal': 'outSignal.nii',
        'outMasked2': 'outMasked2.nii',
        'outMasked': 'outMasked.nii'
    }
    _redirect_x = True


class MedicAlgorithmThresholdToBinaryMaskInputSpec(CommandLineInputSpec):
    inLabel = InputMultiPath(
        File, desc="Input volumes", sep=";", argstr="--inLabel %s")
    inMinimum = traits.Float(
        desc="Minimum threshold value.", argstr="--inMinimum %f")
    inMaximum = traits.Float(
        desc="Maximum threshold value.", argstr="--inMaximum %f")
    inUse = traits.Enum(
        "true",
        "false",
        desc="Use the images max intensity as the max value of the range.",
        argstr="--inUse %s")
    xPrefExt = traits.Enum(
        "nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outBinary = InputMultiPath(
        File, desc="Binary Mask", sep=";", argstr="--outBinary %s")
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d")
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True)


class MedicAlgorithmThresholdToBinaryMaskOutputSpec(TraitedSpec):
    pass


class MedicAlgorithmThresholdToBinaryMask(SEMLikeCommandLine):
    """title: Threshold to Binary Mask

category: Developer Tools

description: Given a volume and an intensity range create a binary mask for values within that range.

version: 1.2.RC

documentation-url: http://www.iacl.ece.jhu.edu/

"""

    input_spec = MedicAlgorithmThresholdToBinaryMaskInputSpec
    output_spec = MedicAlgorithmThresholdToBinaryMaskOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.utilities.volume.MedicAlgorithmThresholdToBinaryMask "
    _outputs_filenames = {}
    _redirect_x = True



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Nipype : Neuroimaging in Python pipelines and interfaces package.

Nipype intends to create python interfaces to other neuroimaging
packages and create an API for specifying a full analysis pipeline in
python.

Much of the machinery at the beginning of this file has been copied over from
nibabel denoted by ## START - COPIED FROM NIBABEL and a corresponding ## END

"""
# Build helper
import sys
from glob import glob
import os
from os.path import join as pjoin
from io import open

# Commit hash writing, and dependency checking
from setuptools.command.build_py import build_py


PY3 = sys.version_info[0] >= 3


class BuildWithCommitInfoCommand(build_py):
    """ Return extended build command class for recording commit

    The extended command tries to run git to find the current commit, getting
    the empty string if it fails.  It then writes the commit hash into a file
    in the `pkg_dir` path, named ``COMMIT_INFO.txt``.

    In due course this information can be used by the package after it is
    installed, to tell you what commit it was installed from if known.

    To make use of this system, you need a package with a COMMIT_INFO.txt file
    e.g. ``myproject/COMMIT_INFO.txt`` - that might well look like this::

        # This is an ini file that may contain information about the code state
        [commit hash]
        # The line below may contain a valid hash if it has been substituted
        # during 'git archive'
        archive_subst_hash=$Format:%h$
        # This line may be modified by the install process
        install_hash=

    The COMMIT_INFO file above is also designed to be used with git
    substitution - so you probably also want a ``.gitattributes`` file in the
    root directory of your working tree that contains something like this::

       myproject/COMMIT_INFO.txt export-subst

    That will cause the ``COMMIT_INFO.txt`` file to get filled in by ``git
    archive`` - useful in case someone makes such an archive - for example with
    via the github 'download source' button.

    Although all the above will work as is, you might consider having something
    like a ``get_info()`` function in your package to display the commit
    information at the terminal.  See the ``pkg_info.py`` module in the nipy
    package for an example.
    """
    def run(self):
        import subprocess
        import configparser

        build_py.run(self)
        proc = subprocess.Popen('git rev-parse --short HEAD',
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                shell=True)
        repo_commit, _ = proc.communicate()
        # Fix for python 3
        if PY3:
            repo_commit = repo_commit.decode()

        # We write the installation commit even if it's empty
        cfg_parser = configparser.RawConfigParser()
        cfg_parser.read(pjoin('nipype', 'COMMIT_INFO.txt'))
        cfg_parser.set('commit hash', 'install_hash', repo_commit.strip())
        out_pth = pjoin(self.build_lib, 'nipype', 'COMMIT_INFO.txt')
        if PY3:
            cfg_parser.write(open(out_pth, 'wt'))
        else:
            cfg_parser.write(open(out_pth, 'wb'))


def main():
    from setuptools import setup, find_packages

    thispath, _ = os.path.split(__file__)

    testdatafiles = [pjoin('testing', 'data', val)
                     for val in os.listdir(pjoin(thispath, 'nipype', 'testing',
                                                 'data'))
                     if not os.path.isdir(pjoin(thispath, 'nipype', 'testing',
                                                'data', val))]

    testdatafiles += [
        pjoin('testing', 'data', 'dicomdir', '*'),
        pjoin('testing', 'data', 'bedpostxout', '*'),
        pjoin('testing', 'data', 'tbss_dir', '*'),
        pjoin('testing', 'data', 'brukerdir', 'fid'),
        pjoin('testing', 'data', 'brukerdir', 'pdata', '1', '*'),
        pjoin('testing', 'data', 'ds005', '*'),
        pjoin('testing', 'data', 'realign_json.json'),
        pjoin('workflows', 'data', '*'),
        pjoin('pipeline', 'engine', 'report_template.html'),
        pjoin('external', 'd3.js'),
        pjoin('interfaces', 'fsl', 'model_templates', '*'),
        pjoin('interfaces', 'tests', 'use_resources'),
        'pytest.ini',
        'conftest.py',
    ]

    # Python 3: use a locals dictionary
    # http://stackoverflow.com/a/1463370/6820620
    ldict = locals()
    # Get version and release info, which is all stored in nipype/info.py
    ver_file = os.path.join(thispath, 'nipype', 'info.py')
    with open(ver_file) as infofile:
        exec(infofile.read(), globals(), ldict)

    SETUP_REQUIRES = ['future']
    if sys.version_info <= (3, 4):
        SETUP_REQUIRES.append('configparser')
    setup(
        name=ldict['NAME'],
        maintainer=ldict['MAINTAINER'],
        maintainer_email=ldict['MAINTAINER_EMAIL'],
        description=ldict['DESCRIPTION'],
        long_description=ldict['LONG_DESCRIPTION'],
        url=ldict['URL'],
        download_url=ldict['DOWNLOAD_URL'],
        license=ldict['LICENSE'],
        classifiers=ldict['CLASSIFIERS'],
        author=ldict['AUTHOR'],
        author_email=ldict['AUTHOR_EMAIL'],
        platforms=ldict['PLATFORMS'],
        version=ldict['VERSION'],
        python_requires=ldict['PYTHON_REQUIRES'],
        install_requires=ldict['REQUIRES'],
        setup_requires=SETUP_REQUIRES,
        provides=ldict['PROVIDES'],
        packages=find_packages(),
        package_data={'nipype': testdatafiles},
        cmdclass={'build_py': BuildWithCommitInfoCommand},
        tests_require=ldict['TESTS_REQUIRES'],
        zip_safe=False,
        extras_require=ldict['EXTRA_REQUIRES'],
        entry_points='''
           [console_scripts]
           nipypecli=nipype.scripts.cli:cli
        '''
    )

if __name__ == "__main__":
    main()



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Smooth


def test_Smooth_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='--i %s',
            extensions=None,
            mandatory=True,
        ),
        num_iters=dict(
            argstr='--niters %d',
            mandatory=True,
            xor=['surface_fwhm'],
        ),
        proj_frac=dict(
            argstr='--projfrac %s',
            xor=['proj_frac_avg'],
        ),
        proj_frac_avg=dict(
            argstr='--projfrac-avg %.2f %.2f %.2f',
            xor=['proj_frac'],
        ),
        reg_file=dict(
            argstr='--reg %s',
            extensions=None,
            mandatory=True,
        ),
        smoothed_file=dict(
            argstr='--o %s',
            extensions=None,
            genfile=True,
        ),
        subjects_dir=dict(),
        surface_fwhm=dict(
            argstr='--fwhm %f',
            mandatory=True,
            requires=['reg_file'],
            xor=['num_iters'],
        ),
        vol_fwhm=dict(argstr='--vol-fwhm %f', ),
    )
    inputs = Smooth.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Smooth_outputs():
    output_map = dict(smoothed_file=dict(extensions=None, ), )
    outputs = Smooth.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The fsl module provides classes for interfacing with the `FSL
<http://www.fmrib.ox.ac.uk/fsl/index.html>`_ command line tools.  This
was written to work with FSL version 5.0.4.
"""
from __future__ import print_function, division, unicode_literals, \
    absolute_import
from builtins import str

import os
import numpy as np
import nibabel as nb
import warnings

from ...utils.filemanip import split_filename
from ...utils import NUMPY_MMAP

from ..base import (traits, TraitedSpec, InputMultiPath, File, isdefined)
from .base import FSLCommand, FSLCommandInputSpec, Info


class PrepareFieldmapInputSpec(FSLCommandInputSpec):
    scanner = traits.String(
        'SIEMENS',
        argstr='%s',
        position=1,
        desc='must be SIEMENS',
        usedefault=True)
    in_phase = File(
        exists=True,
        argstr='%s',
        position=2,
        mandatory=True,
        desc=('Phase difference map, in SIEMENS format range from '
              '0-4096 or 0-8192)'))
    in_magnitude = File(
        exists=True,
        argstr='%s',
        position=3,
        mandatory=True,
        desc='Magnitude difference map, brain extracted')
    delta_TE = traits.Float(
        2.46,
        usedefault=True,
        mandatory=True,
        argstr='%f',
        position=-2,
        desc=('echo time difference of the '
              'fieldmap sequence in ms. (usually 2.46ms in'
              ' Siemens)'))
    nocheck = traits.Bool(
        False,
        position=-1,
        argstr='--nocheck',
        usedefault=True,
        desc=('do not perform sanity checks for image '
              'size/range/dimensions'))
    out_fieldmap = File(
        argstr='%s', position=4, desc='output name for prepared fieldmap')


class PrepareFieldmapOutputSpec(TraitedSpec):
    out_fieldmap = File(exists=True, desc='output name for prepared fieldmap')


class PrepareFieldmap(FSLCommand):
    """
    Interface for the fsl_prepare_fieldmap script (FSL 5.0)

    Prepares a fieldmap suitable for FEAT from SIEMENS data - saves output in
    rad/s format (e.g. ```fsl_prepare_fieldmap SIEMENS
    images_3_gre_field_mapping images_4_gre_field_mapping fmap_rads 2.65```).


    Examples
    --------

    >>> from nipype.interfaces.fsl import PrepareFieldmap
    >>> prepare = PrepareFieldmap()
    >>> prepare.inputs.in_phase = "phase.nii"
    >>> prepare.inputs.in_magnitude = "magnitude.nii"
    >>> prepare.inputs.output_type = "NIFTI_GZ"
    >>> prepare.cmdline # doctest: +ELLIPSIS
    'fsl_prepare_fieldmap SIEMENS phase.nii magnitude.nii \
.../phase_fslprepared.nii.gz 2.460000'
    >>> res = prepare.run() # doctest: +SKIP


    """
    _cmd = 'fsl_prepare_fieldmap'
    input_spec = PrepareFieldmapInputSpec
    output_spec = PrepareFieldmapOutputSpec

    def _parse_inputs(self, skip=None):
        if skip is None:
            skip = []

        if not isdefined(self.inputs.out_fieldmap):
            self.inputs.out_fieldmap = self._gen_fname(
                self.inputs.in_phase, suffix='_fslprepared')

        if not isdefined(self.inputs.nocheck) or not self.inputs.nocheck:
            skip += ['nocheck']

        return super(PrepareFieldmap, self)._parse_inputs(skip=skip)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_fieldmap'] = self.inputs.out_fieldmap
        return outputs

    def _run_interface(self, runtime):
        runtime = super(PrepareFieldmap, self)._run_interface(runtime)

        if runtime.returncode == 0:
            out_file = self.inputs.out_fieldmap
            im = nb.load(out_file, mmap=NUMPY_MMAP)
            dumb_img = nb.Nifti1Image(np.zeros(im.shape), im.affine, im.header)
            out_nii = nb.funcs.concat_images((im, dumb_img))
            nb.save(out_nii, out_file)

        return runtime


class TOPUPInputSpec(FSLCommandInputSpec):
    in_file = File(
        exists=True,
        mandatory=True,
        desc='name of 4D file with images',
        argstr='--imain=%s')
    encoding_file = File(
        exists=True,
        mandatory=True,
        xor=['encoding_direction'],
        desc='name of text file with PE directions/times',
        argstr='--datain=%s')
    encoding_direction = traits.List(
        traits.Enum('y', 'x', 'z', 'x-', 'y-', 'z-'),
        mandatory=True,
        xor=['encoding_file'],
        requires=['readout_times'],
        argstr='--datain=%s',
        desc=('encoding direction for automatic '
              'generation of encoding_file'))
    readout_times = InputMultiPath(
        traits.Float,
        requires=['encoding_direction'],
        xor=['encoding_file'],
        mandatory=True,
        desc=('readout times (dwell times by # '
              'phase-encode steps minus 1)'))
    out_base = File(
        desc=('base-name of output files (spline '
              'coefficients (Hz) and movement parameters)'),
        name_source=['in_file'],
        name_template='%s_base',
        argstr='--out=%s',
        hash_files=False)
    out_field = File(
        argstr='--fout=%s',
        hash_files=False,
        name_source=['in_file'],
        name_template='%s_field',
        desc='name of image file with field (Hz)')
    out_warp_prefix = traits.Str(
        "warpfield",
        argstr='--dfout=%s',
        hash_files=False,
        desc='prefix for the warpfield images (in mm)',
        usedefault=True)
    out_mat_prefix = traits.Str(
        "xfm",
        argstr='--rbmout=%s',
        hash_files=False,
        desc='prefix for the realignment matrices',
        usedefault=True)
    out_jac_prefix = traits.Str(
        "jac",
        argstr='--jacout=%s',
        hash_files=False,
        desc='prefix for the warpfield images',
        usedefault=True)
    out_corrected = File(
        argstr='--iout=%s',
        hash_files=False,
        name_source=['in_file'],
        name_template='%s_corrected',
        desc='name of 4D image file with unwarped images')
    out_logfile = File(
        argstr='--logout=%s',
        desc='name of log-file',
        name_source=['in_file'],
        name_template='%s_topup.log',
        keep_extension=True,
        hash_files=False)

    # TODO: the following traits admit values separated by commas, one value
    # per registration level inside topup.
    warp_res = traits.Float(
        argstr='--warpres=%f',
        desc=('(approximate) resolution (in mm) of warp '
              'basis for the different sub-sampling levels'))
    subsamp = traits.Int(argstr='--subsamp=%d',
                         desc='sub-sampling scheme')
    fwhm = traits.Float(
        argstr='--fwhm=%f',
        desc='FWHM (in mm) of gaussian smoothing kernel')
    config = traits.String(
        'b02b0.cnf',
        argstr='--config=%s',
        usedefault=True,
        desc=('Name of config file specifying command line '
              'arguments'))
    max_iter = traits.Int(
        argstr='--miter=%d',
        desc='max # of non-linear iterations')
    reg_lambda = traits.Float(
        argstr='--lambda=%0.f',
        desc=('Weight of regularisation, default '
              'depending on --ssqlambda and --regmod switches.'))
    ssqlambda = traits.Enum(
        1,
        0,
        argstr='--ssqlambda=%d',
        desc=('Weight lambda by the current value of the '
              'ssd. If used (=1), the effective weight of '
              'regularisation term becomes higher for the '
              'initial iterations, therefore initial steps'
              ' are a little smoother than they would '
              'without weighting. This reduces the '
              'risk of finding a local minimum.'))
    regmod = traits.Enum(
        'bending_energy',
        'membrane_energy',
        argstr='--regmod=%s',
        desc=('Regularisation term implementation. Defaults '
              'to bending_energy. Note that the two functions'
              ' have vastly different scales. The membrane '
              'energy is based on the first derivatives and '
              'the bending energy on the second derivatives. '
              'The second derivatives will typically be much '
              'smaller than the first derivatives, so input '
              'lambda will have to be larger for '
              'bending_energy to yield approximately the same'
              ' level of regularisation.'))
    estmov = traits.Enum(
        1, 0, argstr='--estmov=%d', desc='estimate movements if set')
    minmet = traits.Enum(
        0,
        1,
        argstr='--minmet=%d',
        desc=('Minimisation method 0=Levenberg-Marquardt, '
              '1=Scaled Conjugate Gradient'))
    splineorder = traits.Int(
        argstr='--splineorder=%d',
        desc=('order of spline, 2->Qadratic spline, '
              '3->Cubic spline'))
    numprec = traits.Enum(
        'double',
        'float',
        argstr='--numprec=%s',
        desc=('Precision for representing Hessian, double '
              'or float.'))
    interp = traits.Enum(
        'spline',
        'linear',
        argstr='--interp=%s',
        desc='Image interpolation model, linear or spline.')
    scale = traits.Enum(
        0,
        1,
        argstr='--scale=%d',
        desc=('If set (=1), the images are individually scaled'
              ' to a common mean'))
    regrid = traits.Enum(
        1,
        0,
        argstr='--regrid=%d',
        desc=('If set (=1), the calculations are done in a '
              'different grid'))


class TOPUPOutputSpec(TraitedSpec):
    out_fieldcoef = File(
        exists=True, desc='file containing the field coefficients')
    out_movpar = File(exists=True, desc='movpar.txt output file')
    out_enc_file = File(desc='encoding directions file output for applytopup')
    out_field = File(desc='name of image file with field (Hz)')
    out_warps = traits.List(File(exists=True), desc='warpfield images')
    out_jacs = traits.List(File(exists=True), desc='Jacobian images')
    out_mats = traits.List(File(exists=True), desc='realignment matrices')
    out_corrected = File(desc='name of 4D image file with unwarped images')
    out_logfile = File(desc='name of log-file')


class TOPUP(FSLCommand):
    """
    Interface for FSL topup, a tool for estimating and correcting
    susceptibility induced distortions. See FSL documentation for
    `reference <http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/TOPUP>`_,
    `usage examples
    <http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/topup/ExampleTopupFollowedByApplytopup>`_,
    and `exemplary config files
    <https://github.com/ahheckel/FSL-scripts/blob/master/rsc/fsl/fsl4/topup/b02b0.cnf>`_.

    Examples
    --------

    >>> from nipype.interfaces.fsl import TOPUP
    >>> topup = TOPUP()
    >>> topup.inputs.in_file = "b0_b0rev.nii"
    >>> topup.inputs.encoding_file = "topup_encoding.txt"
    >>> topup.inputs.output_type = "NIFTI_GZ"
    >>> topup.cmdline # doctest: +ELLIPSIS
    'topup --config=b02b0.cnf --datain=topup_encoding.txt \
--imain=b0_b0rev.nii --out=b0_b0rev_base --iout=b0_b0rev_corrected.nii.gz \
--fout=b0_b0rev_field.nii.gz --jacout=jac --logout=b0_b0rev_topup.log \
--rbmout=xfm --dfout=warpfield'
    >>> res = topup.run() # doctest: +SKIP

    """
    _cmd = 'topup'
    input_spec = TOPUPInputSpec
    output_spec = TOPUPOutputSpec

    def _format_arg(self, name, trait_spec, value):
        if name == 'encoding_direction':
            return trait_spec.argstr % self._generate_encfile()
        if name == 'out_base':
            path, name, ext = split_filename(value)
            if path != '':
                if not os.path.exists(path):
                    raise ValueError('out_base path must exist if provided')
        return super(TOPUP, self)._format_arg(name, trait_spec, value)

    def _list_outputs(self):
        outputs = super(TOPUP, self)._list_outputs()
        del outputs['out_base']
        base_path = None
        if isdefined(self.inputs.out_base):
            base_path, base, _ = split_filename(self.inputs.out_base)
            if base_path == '':
                base_path = None
        else:
            base = split_filename(self.inputs.in_file)[1] + '_base'
        outputs['out_fieldcoef'] = self._gen_fname(
            base, suffix='_fieldcoef', cwd=base_path)
        outputs['out_movpar'] = self._gen_fname(
            base, suffix='_movpar', ext='.txt', cwd=base_path)

        n_vols = nb.load(self.inputs.in_file).shape[-1]
        ext = Info.output_type_to_ext(self.inputs.output_type)
        fmt = os.path.abspath('{prefix}_{i:02d}{ext}').format
        outputs['out_warps'] = [
            fmt(prefix=self.inputs.out_warp_prefix, i=i, ext=ext)
            for i in range(1, n_vols + 1)
        ]
        outputs['out_jacs'] = [
            fmt(prefix=self.inputs.out_jac_prefix, i=i, ext=ext)
            for i in range(1, n_vols + 1)
        ]
        outputs['out_mats'] = [
            fmt(prefix=self.inputs.out_mat_prefix, i=i, ext=".mat")
            for i in range(1, n_vols + 1)
        ]

        if isdefined(self.inputs.encoding_direction):
            outputs['out_enc_file'] = self._get_encfilename()
        return outputs

    def _get_encfilename(self):
        out_file = os.path.join(
            os.getcwd(),
            ('%s_encfile.txt' % split_filename(self.inputs.in_file)[1]))
        return out_file

    def _generate_encfile(self):
        """Generate a topup compatible encoding file based on given directions
        """
        out_file = self._get_encfilename()
        durations = self.inputs.readout_times
        if len(self.inputs.encoding_direction) != len(durations):
            if len(self.inputs.readout_times) != 1:
                raise ValueError(('Readout time must be a float or match the'
                                  'length of encoding directions'))
            durations = durations * len(self.inputs.encoding_direction)

        lines = []
        for idx, encdir in enumerate(self.inputs.encoding_direction):
            direction = 1.0
            if encdir.endswith('-'):
                direction = -1.0
            line = [
                float(val[0] == encdir[0]) * direction
                for val in ['x', 'y', 'z']
            ] + [durations[idx]]
            lines.append(line)
        np.savetxt(out_file, np.array(lines), fmt=b'%d %d %d %.8f')
        return out_file

    def _overload_extension(self, value, name=None):
        if name == 'out_base':
            return value
        return super(TOPUP, self)._overload_extension(value, name)


class ApplyTOPUPInputSpec(FSLCommandInputSpec):
    in_files = InputMultiPath(
        File(exists=True),
        mandatory=True,
        desc='name of file with images',
        argstr='--imain=%s',
        sep=',')
    encoding_file = File(
        exists=True,
        mandatory=True,
        desc='name of text file with PE directions/times',
        argstr='--datain=%s')
    in_index = traits.List(
        traits.Int,
        argstr='--inindex=%s',
        sep=',',
        desc='comma separated list of indices corresponding to --datain')
    in_topup_fieldcoef = File(
        exists=True,
        argstr="--topup=%s",
        copyfile=False,
        requires=['in_topup_movpar'],
        desc=('topup file containing the field '
              'coefficients'))
    in_topup_movpar = File(
        exists=True,
        requires=['in_topup_fieldcoef'],
        copyfile=False,
        desc='topup movpar.txt file')
    out_corrected = File(
        desc='output (warped) image',
        name_source=['in_files'],
        name_template='%s_corrected',
        argstr='--out=%s')
    method = traits.Enum(
        'jac',
        'lsr',
        argstr='--method=%s',
        desc=('use jacobian modulation (jac) or least-squares'
              ' resampling (lsr)'))
    interp = traits.Enum(
        'trilinear',
        'spline',
        argstr='--interp=%s',
        desc='interpolation method')
    datatype = traits.Enum(
        'char',
        'short',
        'int',
        'float',
        'double',
        argstr='-d=%s',
        desc='force output data type')


class ApplyTOPUPOutputSpec(TraitedSpec):
    out_corrected = File(
        exists=True, desc=('name of 4D image file with '
                           'unwarped images'))


class ApplyTOPUP(FSLCommand):
    """
    Interface for FSL topup, a tool for estimating and correcting
    susceptibility induced distortions.
    `General reference
    <http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/topup/ApplytopupUsersGuide>`_
    and `use example
    <http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/topup/ExampleTopupFollowedByApplytopup>`_.


    Examples
    --------

    >>> from nipype.interfaces.fsl import ApplyTOPUP
    >>> applytopup = ApplyTOPUP()
    >>> applytopup.inputs.in_files = ["epi.nii", "epi_rev.nii"]
    >>> applytopup.inputs.encoding_file = "topup_encoding.txt"
    >>> applytopup.inputs.in_topup_fieldcoef = "topup_fieldcoef.nii.gz"
    >>> applytopup.inputs.in_topup_movpar = "topup_movpar.txt"
    >>> applytopup.inputs.output_type = "NIFTI_GZ"
    >>> applytopup.cmdline # doctest: +ELLIPSIS
    'applytopup --datain=topup_encoding.txt --imain=epi.nii,epi_rev.nii \
--inindex=1,2 --topup=topup --out=epi_corrected.nii.gz'
    >>> res = applytopup.run() # doctest: +SKIP

    """
    _cmd = 'applytopup'
    input_spec = ApplyTOPUPInputSpec
    output_spec = ApplyTOPUPOutputSpec

    def _parse_inputs(self, skip=None):
        if skip is None:
            skip = []

        # If not defined, assume index are the first N entries in the
        # parameters file, for N input images.
        if not isdefined(self.inputs.in_index):
            self.inputs.in_index = list(
                range(1,
                      len(self.inputs.in_files) + 1))

        return super(ApplyTOPUP, self)._parse_inputs(skip=skip)

    def _format_arg(self, name, spec, value):
        if name == 'in_topup_fieldcoef':
            return spec.argstr % value.split('_fieldcoef')[0]
        return super(ApplyTOPUP, self)._format_arg(name, spec, value)


class EddyInputSpec(FSLCommandInputSpec):
    in_file = File(
        exists=True,
        mandatory=True,
        argstr='--imain=%s',
        desc=('File containing all the images to estimate '
              'distortions for'))
    in_mask = File(
        exists=True,
        mandatory=True,
        argstr='--mask=%s',
        desc='Mask to indicate brain')
    in_index = File(
        exists=True,
        mandatory=True,
        argstr='--index=%s',
        desc=('File containing indices for all volumes in --imain '
              'into --acqp and --topup'))
    in_acqp = File(
        exists=True,
        mandatory=True,
        argstr='--acqp=%s',
        desc='File containing acquisition parameters')
    in_bvec = File(
        exists=True,
        mandatory=True,
        argstr='--bvecs=%s',
        desc=('File containing the b-vectors for all volumes in '
              '--imain'))
    in_bval = File(
        exists=True,
        mandatory=True,
        argstr='--bvals=%s',
        desc=('File containing the b-values for all volumes in '
              '--imain'))
    out_base = traits.Str(
        'eddy_corrected',
        argstr='--out=%s',
        usedefault=True,
        desc=('basename for output (warped) image'))
    session = File(
        exists=True,
        argstr='--session=%s',
        desc=('File containing session indices for all volumes in '
              '--imain'))
    in_topup_fieldcoef = File(
        exists=True,
        argstr="--topup=%s",
        requires=['in_topup_movpar'],
        desc=('topup file containing the field '
              'coefficients'))
    in_topup_movpar = File(
        exists=True,
        requires=['in_topup_fieldcoef'],
        desc='topup movpar.txt file')

    flm = traits.Enum(
        'linear',
        'quadratic',
        'cubic',
        argstr='--flm=%s',
        desc='First level EC model')

    slm = traits.Enum(
        'none',
        'linear',
        'quadratic',
        argstr='--slm=%s',
        desc='Second level EC model')

    fep = traits.Bool(
        False, argstr='--fep', desc='Fill empty planes in x- or y-directions')

    interp = traits.Enum(
        'spline',
        'trilinear',
        argstr='--interp=%s',
        desc='Interpolation model for estimation step')

    nvoxhp = traits.Int(
        1000, usedefault=True,
        argstr='--nvoxhp=%s',
        desc=('# of voxels used to estimate the '
              'hyperparameters'))

    fudge_factor = traits.Float(
        10.0, usedefault=True,
        argstr='--ff=%s',
        desc=('Fudge factor for hyperparameter '
              'error variance'))

    dont_sep_offs_move = traits.Bool(
        False,
        argstr='--dont_sep_offs_move',
        desc=('Do NOT attempt to separate '
              'field offset from subject '
              'movement'))

    dont_peas = traits.Bool(
        False,
        argstr='--dont_peas',
        desc="Do NOT perform a post-eddy alignment of "
        "shells")

    fwhm = traits.Float(
        desc=('FWHM for conditioning filter when estimating '
              'the parameters'),
        argstr='--fwhm=%s')

    niter = traits.Int(5, usedefault=True,
                       argstr='--niter=%s', desc='Number of iterations')

    method = traits.Enum(
        'jac',
        'lsr',
        argstr='--resamp=%s',
        desc=('Final resampling method (jacobian/least '
              'squares)'))
    repol = traits.Bool(
        False, argstr='--repol', desc='Detect and replace outlier slices')
    num_threads = traits.Int(
        1,
        usedefault=True,
        nohash=True,
        desc="Number of openmp threads to use")
    is_shelled = traits.Bool(
        False,
        argstr='--data_is_shelled',
        desc="Override internal check to ensure that "
        "date are acquired on a set of b-value "
        "shells")
    field = traits.Str(
        argstr='--field=%s',
        desc="NonTOPUP fieldmap scaled in Hz - filename has "
        "to be provided without an extension. TOPUP is "
        "strongly recommended")
    field_mat = File(
        exists=True,
        argstr='--field_mat=%s',
        desc="Matrix that specifies the relative locations of "
        "the field specified by --field and first volume "
        "in file --imain")
    use_cuda = traits.Bool(False, desc="Run eddy using cuda gpu")
    cnr_maps = traits.Bool(
        False, desc='Output CNR-Maps', argstr='--cnr_maps', min_ver='5.0.10')
    residuals = traits.Bool(
        False, desc='Output Residuals', argstr='--residuals', min_ver='5.0.10')


class EddyOutputSpec(TraitedSpec):
    out_corrected = File(
        exists=True, desc='4D image file containing all the corrected volumes')
    out_parameter = File(
        exists=True,
        desc=('text file with parameters definining the field and'
              'movement for each scan'))
    out_rotated_bvecs = File(
        exists=True, desc='File containing rotated b-values for all volumes')
    out_movement_rms = File(
        exists=True, desc='Summary of the "total movement" in each volume')
    out_restricted_movement_rms = File(
        exists=True,
        desc=('Summary of the "total movement" in each volume '
              'disregarding translation in the PE direction'))
    out_shell_alignment_parameters = File(
        exists=True,
        desc=('File containing rigid body movement parameters '
              'between the different shells as estimated by a '
              'post-hoc mutual information based registration'))
    out_outlier_report = File(
        exists=True,
        desc=('Text-file with a plain language report on what '
              'outlier slices eddy has found'))
    out_cnr_maps = File(
        exists=True, desc='path/name of file with the cnr_maps')
    out_residuals = File(
        exists=True, desc='path/name of file with the residuals')


class Eddy(FSLCommand):
    """
    Interface for FSL eddy, a tool for estimating and correcting eddy
    currents induced distortions. `User guide
    <http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/Eddy/UsersGuide>`_ and
    `more info regarding acqp file
    <http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/eddy/Faq#How_do_I_know_what_to_put_into_my_--acqp_file>`_.

    Examples
    --------

    >>> from nipype.interfaces.fsl import Eddy
    >>> eddy = Eddy()
    >>> eddy.inputs.in_file = 'epi.nii'
    >>> eddy.inputs.in_mask  = 'epi_mask.nii'
    >>> eddy.inputs.in_index = 'epi_index.txt'
    >>> eddy.inputs.in_acqp  = 'epi_acqp.txt'
    >>> eddy.inputs.in_bvec  = 'bvecs.scheme'
    >>> eddy.inputs.in_bval  = 'bvals.scheme'
    >>> eddy.inputs.use_cuda = True
    >>> eddy.cmdline # doctest: +ELLIPSIS
    'eddy_cuda --ff=10.0 --acqp=epi_acqp.txt --bvals=bvals.scheme \
--bvecs=bvecs.scheme --imain=epi.nii --index=epi_index.txt \
--mask=epi_mask.nii --niter=5 --nvoxhp=1000 --out=.../eddy_corrected'
    >>> eddy.inputs.use_cuda = False
    >>> eddy.cmdline # doctest: +ELLIPSIS
    'eddy_openmp --ff=10.0 --acqp=epi_acqp.txt --bvals=bvals.scheme \
--bvecs=bvecs.scheme --imain=epi.nii --index=epi_index.txt \
--mask=epi_mask.nii --niter=5 --nvoxhp=1000 --out=.../eddy_corrected'
    >>> res = eddy.run() # doctest: +SKIP

    """
    _cmd = 'eddy_openmp'
    input_spec = EddyInputSpec
    output_spec = EddyOutputSpec

    _num_threads = 1

    def __init__(self, **inputs):
        super(Eddy, self).__init__(**inputs)
        self.inputs.on_trait_change(self._num_threads_update, 'num_threads')
        if not isdefined(self.inputs.num_threads):
            self.inputs.num_threads = self._num_threads
        else:
            self._num_threads_update()
        self.inputs.on_trait_change(self._use_cuda, 'use_cuda')
        if isdefined(self.inputs.use_cuda):
            self._use_cuda()

    def _num_threads_update(self):
        self._num_threads = self.inputs.num_threads
        if not isdefined(self.inputs.num_threads):
            if 'OMP_NUM_THREADS' in self.inputs.environ:
                del self.inputs.environ['OMP_NUM_THREADS']
        else:
            self.inputs.environ['OMP_NUM_THREADS'] = str(
                self.inputs.num_threads)

    def _use_cuda(self):
        self._cmd = 'eddy_cuda' if self.inputs.use_cuda else 'eddy_openmp'

    def _run_interface(self, runtime):
        # If 'eddy_openmp' is missing, use 'eddy'
        FSLDIR = os.getenv('FSLDIR', '')
        cmd = self._cmd
        if all((FSLDIR != '', cmd == 'eddy_openmp',
                not os.path.exists(os.path.join(FSLDIR, 'bin', cmd)))):
            self._cmd = 'eddy'
        runtime = super(Eddy, self)._run_interface(runtime)

        # Restore command to avoid side-effects
        self._cmd = cmd
        return runtime

    def _format_arg(self, name, spec, value):
        if name == 'in_topup_fieldcoef':
            return spec.argstr % value.split('_fieldcoef')[0]
        if name == 'out_base':
            return spec.argstr % os.path.abspath(value)
        return super(Eddy, self)._format_arg(name, spec, value)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_corrected'] = os.path.abspath(
            '%s.nii.gz' % self.inputs.out_base)
        outputs['out_parameter'] = os.path.abspath(
            '%s.eddy_parameters' % self.inputs.out_base)

        # File generation might depend on the version of EDDY
        out_rotated_bvecs = os.path.abspath(
            '%s.eddy_rotated_bvecs' % self.inputs.out_base)
        out_movement_rms = os.path.abspath(
            '%s.eddy_movement_rms' % self.inputs.out_base)
        out_restricted_movement_rms = os.path.abspath(
            '%s.eddy_restricted_movement_rms' % self.inputs.out_base)
        out_shell_alignment_parameters = os.path.abspath(
            '%s.eddy_post_eddy_shell_alignment_parameters' %
            self.inputs.out_base)
        out_outlier_report = os.path.abspath(
            '%s.eddy_outlier_report' % self.inputs.out_base)
        if isdefined(self.inputs.cnr_maps) and self.inputs.cnr_maps:
            out_cnr_maps = os.path.abspath(
                '%s.eddy_cnr_maps.nii.gz' % self.inputs.out_base)
            if os.path.exists(out_cnr_maps):
                outputs['out_cnr_maps'] = out_cnr_maps
        if isdefined(self.inputs.residuals) and self.inputs.residuals:
            out_residuals = os.path.abspath(
                '%s.eddy_residuals.nii.gz' % self.inputs.out_base)
            if os.path.exists(out_residuals):
                outputs['out_residuals'] = out_residuals

        if os.path.exists(out_rotated_bvecs):
            outputs['out_rotated_bvecs'] = out_rotated_bvecs
        if os.path.exists(out_movement_rms):
            outputs['out_movement_rms'] = out_movement_rms
        if os.path.exists(out_restricted_movement_rms):
            outputs['out_restricted_movement_rms'] = \
                out_restricted_movement_rms
        if os.path.exists(out_shell_alignment_parameters):
            outputs['out_shell_alignment_parameters'] = \
                out_shell_alignment_parameters
        if os.path.exists(out_outlier_report):
            outputs['out_outlier_report'] = out_outlier_report

        return outputs


class SigLossInputSpec(FSLCommandInputSpec):
    in_file = File(
        mandatory=True, exists=True, argstr='-i %s', desc='b0 fieldmap file')
    out_file = File(
        argstr='-s %s', desc='output signal loss estimate file', genfile=True)

    mask_file = File(exists=True, argstr='-m %s', desc='brain mask file')
    echo_time = traits.Float(argstr='--te=%f', desc='echo time in seconds')
    slice_direction = traits.Enum(
        'x', 'y', 'z', argstr='-d %s', desc='slicing direction')


class SigLossOuputSpec(TraitedSpec):
    out_file = File(exists=True, desc='signal loss estimate file')


class SigLoss(FSLCommand):
    """
    Estimates signal loss from a field map (in rad/s)

    Examples
    --------

    >>> from nipype.interfaces.fsl import SigLoss
    >>> sigloss = SigLoss()
    >>> sigloss.inputs.in_file = "phase.nii"
    >>> sigloss.inputs.echo_time = 0.03
    >>> sigloss.inputs.output_type = "NIFTI_GZ"
    >>> sigloss.cmdline # doctest: +ELLIPSIS
    'sigloss --te=0.030000 -i phase.nii -s .../phase_sigloss.nii.gz'
    >>> res = sigloss.run() # doctest: +SKIP


    """
    input_spec = SigLossInputSpec
    output_spec = SigLossOuputSpec
    _cmd = 'sigloss'

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = self.inputs.out_file
        if ((not isdefined(outputs['out_file']))
                and (isdefined(self.inputs.in_file))):
            outputs['out_file'] = self._gen_fname(
                self.inputs.in_file, suffix='_sigloss')
        return outputs

    def _gen_filename(self, name):
        if name == 'out_file':
            return self._list_outputs()['out_file']
        return None


class EpiRegInputSpec(FSLCommandInputSpec):
    epi = File(
        exists=True,
        argstr='--epi=%s',
        mandatory=True,
        position=-4,
        desc='EPI image')
    t1_head = File(
        exists=True,
        argstr='--t1=%s',
        mandatory=True,
        position=-3,
        desc='wholehead T1 image')
    t1_brain = File(
        exists=True,
        argstr='--t1brain=%s',
        mandatory=True,
        position=-2,
        desc='brain extracted T1 image')
    out_base = traits.String(
        "epi2struct",
        desc='output base name',
        argstr='--out=%s',
        position=-1,
        usedefault=True)
    fmap = File(
        exists=True, argstr='--fmap=%s', desc='fieldmap image (in rad/s)')
    fmapmag = File(
        exists=True,
        argstr='--fmapmag=%s',
        desc='fieldmap magnitude image - wholehead')
    fmapmagbrain = File(
        exists=True,
        argstr='--fmapmagbrain=%s',
        desc='fieldmap magnitude image - brain extracted')
    wmseg = File(
        exists=True,
        argstr='--wmseg=%s',
        desc='white matter segmentation of T1 image, has to be named \
                 like the t1brain and end on _wmseg')
    echospacing = traits.Float(
        argstr='--echospacing=%f',
        desc='Effective EPI echo spacing  \
                               (sometimes called dwell time) - in seconds')
    pedir = traits.Enum(
        'x',
        'y',
        'z',
        '-x',
        '-y',
        '-z',
        argstr='--pedir=%s',
        desc='phase encoding direction, dir = x/y/z/-x/-y/-z')

    weight_image = File(
        exists=True,
        argstr='--weight=%s',
        desc='weighting image (in T1 space)')
    no_fmapreg = traits.Bool(
        False,
        argstr='--nofmapreg',
        desc='do not perform registration of fmap to T1 \
                        (use if fmap already registered)')
    no_clean = traits.Bool(
        True,
        argstr='--noclean',
        usedefault=True,
        desc='do not clean up intermediate files')


class EpiRegOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='unwarped and coregistered epi input')
    out_1vol = File(
        exists=True, desc='unwarped and coregistered single volume')
    fmap2str_mat = File(
        exists=True, desc='rigid fieldmap-to-structural transform')
    fmap2epi_mat = File(exists=True, desc='rigid fieldmap-to-epi transform')
    fmap_epi = File(exists=True, desc='fieldmap in epi space')
    fmap_str = File(exists=True, desc='fieldmap in structural space')
    fmapmag_str = File(
        exists=True, desc='fieldmap magnitude image in structural space')
    epi2str_inv = File(exists=True, desc='rigid structural-to-epi transform')
    epi2str_mat = File(exists=True, desc='rigid epi-to-structural transform')
    shiftmap = File(exists=True, desc='shiftmap in epi space')
    fullwarp = File(
        exists=True,
        desc='warpfield to unwarp epi and transform into \
                    structural space')
    wmseg = File(
        exists=True, desc='white matter segmentation used in flirt bbr')
    seg = File(
        exists=True, desc='white matter, gray matter, csf segmentation')
    wmedge = File(exists=True, desc='white matter edges for visualization')


class EpiReg(FSLCommand):
    """

    Runs FSL epi_reg script for simultaneous coregistration and fieldmap
    unwarping.

    Examples
    --------

    >>> from nipype.interfaces.fsl import EpiReg
    >>> epireg = EpiReg()
    >>> epireg.inputs.epi='epi.nii'
    >>> epireg.inputs.t1_head='T1.nii'
    >>> epireg.inputs.t1_brain='T1_brain.nii'
    >>> epireg.inputs.out_base='epi2struct'
    >>> epireg.inputs.fmap='fieldmap_phase_fslprepared.nii'
    >>> epireg.inputs.fmapmag='fieldmap_mag.nii'
    >>> epireg.inputs.fmapmagbrain='fieldmap_mag_brain.nii'
    >>> epireg.inputs.echospacing=0.00067
    >>> epireg.inputs.pedir='y'
    >>> epireg.cmdline # doctest: +ELLIPSIS
    'epi_reg --echospacing=0.000670 --fmap=fieldmap_phase_fslprepared.nii \
--fmapmag=fieldmap_mag.nii --fmapmagbrain=fieldmap_mag_brain.nii --noclean \
--pedir=y --epi=epi.nii --t1=T1.nii --t1brain=T1_brain.nii --out=epi2struct'
    >>> epireg.run() # doctest: +SKIP

    """
    _cmd = 'epi_reg'
    input_spec = EpiRegInputSpec
    output_spec = EpiRegOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = os.path.join(os.getcwd(),
                                           self.inputs.out_base + '.nii.gz')
        if (not (isdefined(self.inputs.no_fmapreg) and self.inputs.no_fmapreg)
                and isdefined(self.inputs.fmap)):
            outputs['out_1vol'] = os.path.join(
                os.getcwd(), self.inputs.out_base + '_1vol.nii.gz')
            outputs['fmap2str_mat'] = os.path.join(
                os.getcwd(), self.inputs.out_base + '_fieldmap2str.mat')
            outputs['fmap2epi_mat'] = os.path.join(
                os.getcwd(), self.inputs.out_base + '_fieldmaprads2epi.mat')
            outputs['fmap_epi'] = os.path.join(
                os.getcwd(), self.inputs.out_base + '_fieldmaprads2epi.nii.gz')
            outputs['fmap_str'] = os.path.join(
                os.getcwd(), self.inputs.out_base + '_fieldmaprads2str.nii.gz')
            outputs['fmapmag_str'] = os.path.join(
                os.getcwd(), self.inputs.out_base + '_fieldmap2str.nii.gz')
            outputs['shiftmap'] = os.path.join(
                os.getcwd(),
                self.inputs.out_base + '_fieldmaprads2epi_shift.nii.gz')
            outputs['fullwarp'] = os.path.join(
                os.getcwd(), self.inputs.out_base + '_warp.nii.gz')
            outputs['epi2str_inv'] = os.path.join(
                os.getcwd(), self.inputs.out_base + '_inv.mat')

        outputs['epi2str_mat'] = os.path.join(os.getcwd(),
                                              self.inputs.out_base + '.mat')
        outputs['wmedge'] = os.path.join(
            os.getcwd(), self.inputs.out_base + '_fast_wmedge.nii.gz')
        outputs['wmseg'] = os.path.join(
            os.getcwd(), self.inputs.out_base + '_fast_wmseg.nii.gz')
        outputs['seg'] = os.path.join(
            os.getcwd(), self.inputs.out_base + '_fast_seg.nii.gz')

        return outputs


#######################################
# deprecated interfaces
#######################################


class EPIDeWarpInputSpec(FSLCommandInputSpec):
    mag_file = File(
        exists=True,
        desc='Magnitude file',
        argstr='--mag %s',
        position=0,
        mandatory=True)
    dph_file = File(
        exists=True,
        desc='Phase file assumed to be scaled from 0 to 4095',
        argstr='--dph %s',
        mandatory=True)
    exf_file = File(
        exists=True,
        desc='example func volume (or use epi)',
        argstr='--exf %s')
    epi_file = File(
        exists=True, desc='EPI volume to unwarp', argstr='--epi %s')
    tediff = traits.Float(
        2.46,
        usedefault=True,
        desc='difference in B0 field map TEs',
        argstr='--tediff %s')
    esp = traits.Float(
        0.58, desc='EPI echo spacing', argstr='--esp %s', usedefault=True)
    sigma = traits.Int(
        2,
        usedefault=True,
        argstr='--sigma %s',
        desc="2D spatial gaussing smoothing \
                       stdev (default = 2mm)")
    vsm = traits.String(
        genfile=True, desc='voxel shift map', argstr='--vsm %s')
    exfdw = traits.String(
        desc='dewarped example func volume', genfile=True, argstr='--exfdw %s')
    epidw = traits.String(
        desc='dewarped epi volume', genfile=False, argstr='--epidw %s')
    tmpdir = traits.String(genfile=True, desc='tmpdir', argstr='--tmpdir %s')
    nocleanup = traits.Bool(
        True, usedefault=True, desc='no cleanup', argstr='--nocleanup')
    cleanup = traits.Bool(desc='cleanup', argstr='--cleanup')


class EPIDeWarpOutputSpec(TraitedSpec):
    unwarped_file = File(desc="unwarped epi file")
    vsm_file = File(desc="voxel shift map")
    exfdw = File(desc="dewarped functional volume example")
    exf_mask = File(desc="Mask from example functional volume")


class EPIDeWarp(FSLCommand):
    """
    Wraps the unwarping script `epidewarp.fsl
    <http://surfer.nmr.mgh.harvard.edu/fswiki/epidewarp.fsl>`_.

    .. warning:: deprecated in FSL, please use
      :func:`niflow.nipype1.workflows.dmri.preprocess.epi.sdc_fmb` instead.

    Examples
    --------

    >>> from nipype.interfaces.fsl import EPIDeWarp
    >>> dewarp = EPIDeWarp()
    >>> dewarp.inputs.epi_file = "functional.nii"
    >>> dewarp.inputs.mag_file = "magnitude.nii"
    >>> dewarp.inputs.dph_file = "phase.nii"
    >>> dewarp.inputs.output_type = "NIFTI_GZ"
    >>> dewarp.cmdline # doctest: +ELLIPSIS
    'epidewarp.fsl --mag magnitude.nii --dph phase.nii --epi functional.nii \
--esp 0.58 --exfdw .../exfdw.nii.gz --nocleanup --sigma 2 --tediff 2.46 \
--tmpdir .../temp --vsm .../vsm.nii.gz'
    >>> res = dewarp.run() # doctest: +SKIP


    """
    _cmd = 'epidewarp.fsl'
    input_spec = EPIDeWarpInputSpec
    output_spec = EPIDeWarpOutputSpec

    def __init__(self, **inputs):
        warnings.warn(("Deprecated: Please use "
                       "nipype.workflows.dmri.preprocess.epi.sdc_fmb instead"),
                      DeprecationWarning)
        return super(EPIDeWarp, self).__init__(**inputs)

    def _run_interface(self, runtime):
        runtime = super(EPIDeWarp, self)._run_interface(runtime)
        if runtime.stderr:
            self.raise_exception(runtime)
        return runtime

    def _gen_filename(self, name):
        if name == 'exfdw':
            if isdefined(self.inputs.exf_file):
                return self._gen_fname(self.inputs.exf_file, suffix="_exfdw")
            else:
                return self._gen_fname("exfdw")
        if name == 'epidw':
            if isdefined(self.inputs.epi_file):
                return self._gen_fname(self.inputs.epi_file, suffix="_epidw")
        if name == 'vsm':
            return self._gen_fname('vsm')
        if name == 'tmpdir':
            return os.path.join(os.getcwd(), 'temp')
        return None

    def _list_outputs(self):
        outputs = self.output_spec().get()
        if not isdefined(self.inputs.exfdw):
            outputs['exfdw'] = self._gen_filename('exfdw')
        else:
            outputs['exfdw'] = self.inputs.exfdw
        if isdefined(self.inputs.epi_file):
            if isdefined(self.inputs.epidw):
                outputs['unwarped_file'] = self.inputs.epidw
            else:
                outputs['unwarped_file'] = self._gen_filename('epidw')
        if not isdefined(self.inputs.vsm):
            outputs['vsm_file'] = self._gen_filename('vsm')
        else:
            outputs['vsm_file'] = self._gen_fname(self.inputs.vsm)
        if not isdefined(self.inputs.tmpdir):
            outputs['exf_mask'] = self._gen_fname(
                cwd=self._gen_filename('tmpdir'), basename='maskexf')
        else:
            outputs['exf_mask'] = self._gen_fname(
                cwd=self.inputs.tmpdir, basename='maskexf')
        return outputs


class EddyCorrectInputSpec(FSLCommandInputSpec):
    in_file = File(
        exists=True,
        desc='4D input file',
        argstr='%s',
        position=0,
        mandatory=True)
    out_file = File(
        desc='4D output file',
        argstr='%s',
        position=1,
        name_source=['in_file'],
        name_template='%s_edc',
        output_name='eddy_corrected')
    ref_num = traits.Int(
        0,
        argstr='%d',
        position=2,
        desc='reference number',
        mandatory=True,
        usedefault=True)


class EddyCorrectOutputSpec(TraitedSpec):
    eddy_corrected = File(
        exists=True, desc='path/name of 4D eddy corrected output file')


class EddyCorrect(FSLCommand):
    """

    .. warning:: Deprecated in FSL. Please use
      :class:`nipype.interfaces.fsl.epi.Eddy` instead

    Example
    -------

    >>> from nipype.interfaces.fsl import EddyCorrect
    >>> eddyc = EddyCorrect(in_file='diffusion.nii',
    ...                     out_file="diffusion_edc.nii", ref_num=0)
    >>> eddyc.cmdline
    'eddy_correct diffusion.nii diffusion_edc.nii 0'

    """
    _cmd = 'eddy_correct'
    input_spec = EddyCorrectInputSpec
    output_spec = EddyCorrectOutputSpec

    def __init__(self, **inputs):
        warnings.warn(("Deprecated: Please use nipype.interfaces.fsl.epi.Eddy "
                       "instead"), DeprecationWarning)
        return super(EddyCorrect, self).__init__(**inputs)

    def _run_interface(self, runtime):
        runtime = super(EddyCorrect, self)._run_interface(runtime)
        if runtime.stderr:
            self.raise_exception(runtime)
        return runtime


class EddyQuadInputSpec(FSLCommandInputSpec):
    base_name = traits.Str(
        'eddy_corrected',
        usedefault=True,
        argstr='%s',
        desc=("Basename (including path) for EDDY output files, i.e., "
              "corrected images and QC files"),
        position=0,
    )
    idx_file = File(
        exists=True,
        mandatory=True,
        argstr="--eddyIdx %s",
        desc=("File containing indices for all volumes into acquisition "
              "parameters")
    )
    param_file = File(
        exists=True,
        mandatory=True,
        argstr="--eddyParams %s",
        desc="File containing acquisition parameters"
    )
    mask_file = File(
        exists=True,
        mandatory=True,
        argstr="--mask %s",
        desc="Binary mask file"
    )
    bval_file = File(
        exists=True,
        mandatory=True,
        argstr="--bvals %s",
        desc="b-values file"
    )
    bvec_file = File(
        exists=True,
        argstr="--bvecs %s",
        desc=("b-vectors file - only used when <base_name>.eddy_residuals "
              "file is present")
    )
    output_dir = traits.Str(
        name_template='%s.qc',
        name_source=['base_name'],
        argstr='--output-dir %s',
        desc="Output directory - default = '<base_name>.qc'",
    )
    field = File(
        exists=True,
        argstr='--field %s',
        desc="TOPUP estimated field (in Hz)",
    )
    slice_spec = File(
        exists=True,
        argstr='--slspec %s',
        desc="Text file specifying slice/group acquisition",
    )
    verbose = traits.Bool(
        argstr='--verbose',
        desc="Display debug messages",
    )


class EddyQuadOutputSpec(TraitedSpec):
    qc_json = File(
        exists=True,
        desc=("Single subject database containing quality metrics and data "
              "info.")
    )
    qc_pdf = File(
        exists=True,
        desc="Single subject QC report."
    )
    avg_b_png = traits.List(
        File(exists=True),
        desc=("Image showing mid-sagittal, -coronal and -axial slices of "
              "each averaged b-shell volume.")
    )
    avg_b0_pe_png = traits.List(
        File(exists=True),
        desc=("Image showing mid-sagittal, -coronal and -axial slices of "
              "each averaged pe-direction b0 volume. Generated when using "
              "the -f option.")
    )
    cnr_png = traits.List(
        File(exists=True),
        desc=("Image showing mid-sagittal, -coronal and -axial slices of "
              "each b-shell CNR volume. Generated when CNR maps are "
              "available.")
    )
    vdm_png = File(
        exists=True,
        desc=("Image showing mid-sagittal, -coronal and -axial slices of "
              "the voxel displacement map. Generated when using the -f "
              "option.")
    )
    residuals = File(
        exists=True,
        desc=("Text file containing the volume-wise mask-averaged squared "
              "residuals. Generated when residual maps are available.")
    )
    clean_volumes = File(
        exists=True,
        desc=("Text file containing a list of clean volumes, based on "
              "the eddy squared residuals. To generate a version of the "
              "pre-processed dataset without outlier volumes, use: "
              "`fslselectvols -i <eddy_corrected_data> -o "
              "eddy_corrected_data_clean --vols=vols_no_outliers.txt`")
    )


class EddyQuad(FSLCommand):
    """
    Interface for FSL eddy_quad, a tool for generating single subject reports
    and storing the quality assessment indices for each subject.
    `User guide <https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/eddyqc/UsersGuide>`_

    Examples
    --------

    >>> from nipype.interfaces.fsl import EddyQuad
    >>> quad = EddyQuad()
    >>> quad.inputs.base_name  = 'eddy_corrected'
    >>> quad.inputs.idx_file   = 'epi_index.txt'
    >>> quad.inputs.param_file = 'epi_acqp.txt'
    >>> quad.inputs.mask_file  = 'epi_mask.nii'
    >>> quad.inputs.bval_file  = 'bvals.scheme'
    >>> quad.inputs.bvec_file  = 'bvecs.scheme'
    >>> quad.inputs.output_dir = 'eddy_corrected.qc'
    >>> quad.inputs.field      = 'fieldmap_phase_fslprepared.nii'
    >>> quad.inputs.verbose    = True
    >>> quad.cmdline
    'eddy_quad eddy_corrected --bvals bvals.scheme --bvecs bvecs.scheme \
--field fieldmap_phase_fslprepared.nii --eddyIdx epi_index.txt \
--mask epi_mask.nii --output-dir eddy_corrected.qc --eddyParams epi_acqp.txt \
--verbose'
    >>> res = quad.run() # doctest: +SKIP

    """
    _cmd = 'eddy_quad'
    input_spec = EddyQuadInputSpec
    output_spec = EddyQuadOutputSpec

    def _list_outputs(self):
        from glob import glob
        outputs = self.output_spec().get()

        # If the output directory isn't defined, the interface seems to use
        # the default but not set its value in `self.inputs.output_dir`
        if not isdefined(self.inputs.output_dir):
            out_dir = os.path.abspath(os.path.basename(self.inputs.base_name) + '.qc')
        else:
            out_dir = os.path.abspath(self.inputs.output_dir)

        outputs['qc_json'] = os.path.join(out_dir, 'qc.json')
        outputs['qc_pdf'] = os.path.join(out_dir, 'qc.pdf')

        # Grab all b* files here. This will also grab the b0_pe* files
        # as well, but only if the field input was provided. So we'll remove
        # them later in the next conditional.
        outputs['avg_b_png'] = sorted(glob(
            os.path.join(out_dir, 'avg_b*.png')
        ))

        if isdefined(self.inputs.field):
            outputs['avg_b0_pe_png'] = sorted(glob(
                os.path.join(out_dir, 'avg_b0_pe*.png')
            ))

            # The previous glob for `avg_b_png` also grabbed the
            # `avg_b0_pe_png` files so we have to remove them
            # from `avg_b_png`.
            for fname in outputs['avg_b0_pe_png']:
                outputs['avg_b_png'].remove(fname)

            outputs['vdm_png'] = os.path.join(out_dir, 'vdm.png')

        outputs['cnr_png'] = sorted(glob(os.path.join(out_dir, 'cnr*.png')))

        residuals = os.path.join(out_dir, 'eddy_msr.txt')
        if os.path.isfile(residuals):
            outputs['residuals'] = residuals

        clean_volumes = os.path.join(out_dir, 'vols_no_outliers.txt')
        if os.path.isfile(clean_volumes):
            outputs['clean_volumes'] = clean_volumes

        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import Threshold


def test_Threshold_inputs():
    input_map = dict(
        contrast_index=dict(mandatory=True, ),
        extent_fdr_p_threshold=dict(usedefault=True, ),
        extent_threshold=dict(usedefault=True, ),
        force_activation=dict(usedefault=True, ),
        height_threshold=dict(usedefault=True, ),
        height_threshold_type=dict(usedefault=True, ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        paths=dict(),
        spm_mat_file=dict(
            copyfile=True,
            extensions=None,
            mandatory=True,
        ),
        stat_image=dict(
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        use_fwe_correction=dict(usedefault=True, ),
        use_mcr=dict(),
        use_topo_fdr=dict(usedefault=True, ),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
    )
    inputs = Threshold.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Threshold_outputs():
    output_map = dict(
        activation_forced=dict(),
        cluster_forming_thr=dict(),
        n_clusters=dict(),
        pre_topo_fdr_map=dict(extensions=None, ),
        pre_topo_n_clusters=dict(),
        thresholded_map=dict(extensions=None, ),
    )
    outputs = Threshold.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



#!/usr/bin/env python3
"""Update and sort the creators list of the zenodo record."""
import sys
import shutil
from pathlib import Path
import json
from fuzzywuzzy import fuzz, process
import subprocess as sp

# These names should go last
CREATORS_LAST = ['Gorgolewski, Krzysztof J.', 'Ghosh, Satrajit']

if __name__ == '__main__':
    contrib_file = Path('line-contributors.txt')
    lines = []
    if contrib_file.exists():
        print('WARNING: Reusing existing line-contributors.txt file.', file=sys.stderr)
        lines = contrib_file.read_text().splitlines()

    git_line_summary_path = shutil.which('git-line-summary')
    if not lines and git_line_summary_path:
        print("Running git-line-summary on nipype repo")
        lines = sp.check_output([git_line_summary_path]).decode().splitlines()
        contrib_file.write_text('\n'.join(lines))

    if not lines:
        raise RuntimeError("""\
Could not find line-contributors from git repository.%s""" % """ \
git-line-summary not found, please install git-extras. """ * (git_line_summary_path is None))

    data = [' '.join(line.strip().split()[1:-1]) for line in lines if '%' in line]

    # load zenodo from master
    zenodo_file = Path('.zenodo.json')
    zenodo = json.loads(zenodo_file.read_text())
    zen_names = [' '.join(val['name'].split(',')[::-1]).strip()
                 for val in zenodo['creators']]
    total_names = len(zen_names)

    name_matches = []
    position = 1
    for ele in data:
        matches = process.extract(ele, zen_names, scorer=fuzz.token_sort_ratio,
                                  limit=2)
        # matches is a list [('First match', % Match), ('Second match', % Match)]
        if matches[0][1] <= 80:
            # skip unmatched names
            print("No entry to sort:", ele)
            continue

        idx = zen_names.index(matches[0][0])
        val = zenodo['creators'][idx]

        if val not in name_matches:
            if val['name'] not in CREATORS_LAST:
                val['position'] = position
                position += 1
            else:
                val['position'] = total_names + CREATORS_LAST.index(val['name'])
            name_matches.append(val)

    for missing in zenodo['creators']:
        if 'position' not in missing:
            missing['position'] = position
            position += 1
            name_matches.append(missing)

    zenodo['creators'] = sorted(name_matches, key=lambda k: k['position'])
    # Remove position
    for creator in zenodo['creators']:
        del creator['position']

    zenodo_file.write_text('%s\n' % json.dumps(zenodo, indent=2, sort_keys=True))



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..svm import SVMTrain


def test_SVMTrain_inputs():
    input_map = dict(
        alphas=dict(
            argstr='-alpha %s',
            extensions=None,
            name_source='in_file',
            name_template='%s_alphas',
            suffix='_alphas',
        ),
        args=dict(argstr='%s', ),
        censor=dict(
            argstr='-censor %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-trainvol %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        kernel=dict(argstr='-kernel %s', ),
        mask=dict(
            argstr='-mask %s',
            copyfile=False,
            extensions=None,
            position=-1,
        ),
        max_iterations=dict(argstr='-max_iterations %d', ),
        model=dict(
            argstr='-model %s',
            extensions=None,
            name_source='in_file',
            name_template='%s_model',
            suffix='_model',
        ),
        nomodelmask=dict(argstr='-nomodelmask', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        options=dict(argstr='%s', ),
        out_file=dict(
            argstr='-bucket %s',
            extensions=None,
            name_source='in_file',
            name_template='%s_vectors',
            suffix='_bucket',
        ),
        outputtype=dict(),
        trainlabels=dict(
            argstr='-trainlabels %s',
            extensions=None,
        ),
        ttype=dict(
            argstr='-type %s',
            mandatory=True,
        ),
        w_out=dict(argstr='-wout', ),
    )
    inputs = SVMTrain.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SVMTrain_outputs():
    output_map = dict(
        alphas=dict(extensions=None, ),
        model=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
    )
    outputs = SVMTrain.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
